<?php

namespace Teknisa\DF\Modules\Entrada\Factories;

use Doctrine\ORM\EntityManager;

use Teknisa\Libs\Util\DateUtil;
use Geral\Entities\Itemproduto;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Contapagar;
use Teknisa\Model\Suprimentos\Entities\Nfrelacaf;
use Indeterminado\Entities\Prodfornxml;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Dfe11000NfEntradaFactory {

    private $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        date_default_timezone_set('America/Sao_Paulo');
    }

    /*************** Formata a data de forma que possa ser inserida no banco *******************/
    public static function formataData($data) {
        if (!$data || !is_string($data))
            return $data;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

    /********************************* Atualização AF *****************************************/
    public function atualizaStatusAF(
        Itemproduto $itemproduto, $status, $oldQtitemprod = null,
        $oldCdprodemis = null, $oldCdproduto = null, $oldCdmarca = null
    ) {
        try {
            $filial     = $itemproduto->getCdfilial();
            $lancamento = $itemproduto->getNrlanctonf();
            $seqItem    = $itemproduto->getNrseqitprod();
            $cdprodemis = $itemproduto->getCdprodemis();
            $cdproduto  = $itemproduto->getCdproduto();
            $cdmarca    = $itemproduto->getCdmarca();
            $qtitemprod = $itemproduto->getQtitemprod();

            $oldCdproduto   = !$oldCdproduto  ? $cdproduto  : $oldCdproduto;
            $oldCdprodemis  = !$oldCdprodemis ? $cdprodemis : $oldCdprodemis;
            $oldCdmarca     = !$oldCdmarca    ? $cdmarca    : $oldCdmarca;
            $oldQtitemprod  = !$oldQtitemprod ? $qtitemprod : $oldQtitemprod;
            
            $montaParamsBaixaAF = $this->montaParamsBaixaAF(
                $filial, $lancamento, $seqItem, $status, $cdprodemis,
                $oldCdprodemis, $cdproduto, $oldCdproduto, $cdmarca,
                $oldCdmarca, $qtitemprod, $oldQtitemprod
            );
            $this->callProcedureBaixaAF($montaParamsBaixaAF);

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function excluirItemAf($item) {
        try {
            $filial     = $item["CDFILIAL"];
            $lancamento = $item["NRLANCTONF"];
            $seqItem    = $item["NRSEQITPROD"];
            $cdprodemis = $item["CDPRODEMIS"];
            $cdproduto  = $item["CDPRODUTO"];
            $cdmarca    = $item["CDMARCA"];
            $qtitemprod = $item["QTITEMPROD"];

            $oldCdproduto   = $cdproduto;
            $oldCdprodemis  = $cdprodemis;
            $oldCdmarca     = $cdmarca   ;
            $oldQtitemprod  = $qtitemprod;
            
            $status = 'E';

            $montaParamsBaixaAF = $this->montaParamsBaixaAF(
                $filial, $lancamento, $seqItem, $status, $cdprodemis,
                $oldCdprodemis, $cdproduto, $oldCdproduto, $cdmarca,
                $oldCdmarca, $qtitemprod, $oldQtitemprod
            );
            $this->callProcedureBaixaAF($montaParamsBaixaAF);
            
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function montaParamsBaixaAF(
        $filial, $lancamento, $seqItem, $status, $cdprodemis,
        $oldCdprodemis, $cdproduto, $oldCdproduto, $cdmarca,
        $oldCdmarca, $qtitemprod, $oldQtitemprod
    ) {
        try {
            $paramsBaixaAF = array(
                'CDFILIAL' => $filial,
                'NRLANCTONF' => $lancamento,
                'NRSEQITPROD' => $seqItem,
                'IDSTATUS' => $status,
                'NEWCDPRODEMIS' => $cdprodemis,
                'OLDCDPRODEMIS' => $oldCdprodemis,
                'NEWCDPRODUTO' => $cdproduto,
                'OLDCDPRODUTO' => $oldCdproduto,
                'NEWCDMARCA' => $cdmarca,
                'OLDCDMARCA' => $oldCdmarca,
                'NEWQTITEMPROD' => $qtitemprod,
                'OLDQTITEMPROD' => $oldQtitemprod
            );
            return $paramsBaixaAF;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * PROCEDURE -> NF_BAIXA_AF
     * P_CDFILIAL      - Código da Filial de lançamento da nota fiscal.
     * P_NRLANCTONF    - Número de lançamento da nota fiscal.
     * P_NRSEQITPROD   - Sequencial do item da nota fiscal
     * P_IDSTATUS      - Status (I - Inserção, A - Alteração, E - Exclusão)
     * P_NEWCDPRODEMIS - Novo código do produto de emissão
     * P_OLDCDPRODEMIS - Antigo código do produto de emissão
     * P_NEWCDPRODUTO  - Novo código do produto
     * P_OLDCDPRODUTO  - Antigo código do produto
     * P_NEWCDMARCA    - Novo código da marca do produto
     * P_OLDCDMARCA    - Antigo código da marca do produto
     * P_NEWQTITEMPROD - Nova quantidade do produto
     * P_OLDQTITEMPROD - Antiga quantidade do produto
     * P_IDERRO        - Identificador de existência de erro
     */
    public function callProcedureBaixaAF($paramsBaixaAF) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'NF_BAIXA_AF');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $paramsBaixaAF['CDFILIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $paramsBaixaAF['NRLANCTONF'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQITPROD', Param::PARAM_INPUT, $paramsBaixaAF['NRSEQITPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDSTATUS', Param::PARAM_INPUT, $paramsBaixaAF['IDSTATUS'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NEWCDPRODEMIS', Param::PARAM_INPUT, $paramsBaixaAF['NEWCDPRODEMIS'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_OLDCDPRODEMIS', Param::PARAM_INPUT, $paramsBaixaAF['OLDCDPRODEMIS'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NEWCDPRODUTO', Param::PARAM_INPUT, $paramsBaixaAF['NEWCDPRODUTO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_OLDCDPRODUTO', Param::PARAM_INPUT, $paramsBaixaAF['OLDCDPRODUTO'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NEWCDMARCA', Param::PARAM_INPUT, $paramsBaixaAF['NEWCDMARCA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_OLDCDMARCA', Param::PARAM_INPUT, $paramsBaixaAF['OLDCDMARCA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NEWQTITEMPROD', Param::PARAM_INPUT, $paramsBaixaAF['NEWQTITEMPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_OLDQTITEMPROD', Param::PARAM_INPUT, $paramsBaixaAF['OLDQTITEMPROD'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDERRO', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 1));
            $erro = $procedure->execute();
            return $erro;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /********************************* / Atualizacao AF *****************************************/

    /******************************* Relacionamento Af/Nf ***************************************/
    public function novoRelacionamentoAfNf(
        $filial, $lancamento, $filialDestAf, $filialLanc, $nrAutoForn,
        $cdmarca, $cdproduto, $dtProgentAf, $qtnfrelacaf, $vrnfrelacaf,
        $qtnfefetaf, $organizacao, $operador
    ) {
        try {
            $data = DateUtil::getDataAtual();
            
            $relacionamentosAnteriores = $this->getNfRelacAf($filialDestAf, $filialLanc, $nrAutoForn, $cdproduto);
            
            foreach($relacionamentosAnteriores as $relacionamento) {
                $qtnfrelacaf -= $relacionamento["QTEFETENTRAF"];
            }
            
            if($qtnfrelacaf < 0) {
                $qtnfrelacaf = 0;
            }
            
            $nfrelacaf = new Nfrelacaf();
            $nfrelacaf->setCdfilial($filial);
            $nfrelacaf->setNrlanctonf($lancamento);
            $nfrelacaf->setCdfildest($filialDestAf);
            $nfrelacaf->setCdfillanc($filialLanc);
            $nfrelacaf->setNrautoforn($nrAutoForn);
            $nfrelacaf->setCdmarca($cdmarca);
            $nfrelacaf->setCdproduto($cdproduto);
            $nfrelacaf->setDtprogentaf(self::formataData($dtProgentAf));
            $nfrelacaf->setQtnfrelacaf($qtnfrelacaf);
            $nfrelacaf->setVrnfrelacaf($vrnfrelacaf);
            // $nfrelacaf->setQtnfefetaf($qtnfefetaf);

            $nfrelacaf->setNrorg($organizacao);
            $nfrelacaf->setNrorginclusao($organizacao);
            $nfrelacaf->setNrorgultatu($organizacao);
            $nfrelacaf->setCdoperinclusao($operador);
            $nfrelacaf->setCdoperultatu($operador);
            $nfrelacaf->setDtinclusao($data);
            $nfrelacaf->setDtultatu($data);

            $this->entityManager->persist($nfrelacaf);

            return $nfrelacaf;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getNfRelacAf($filialDestAf, $filialLanc, $nrAutoForn, $cdproduto) {
        $sql = "SELECT I.QTPROGENTAF, I.QTEFETENTRAF, I.DTPROGENTAF
            FROM ITEMAUTF I
            WHERE NRAUTOFORN = :P_NRAUTOFORN
            AND CDFILDEST = :P_CDFILDEST
            AND CDFILLANC = :P_CDFILLANC
            AND CDPRODUTO = :P_CDPRODUTO";
            
        $params = array("P_CDFILDEST"   => $filialDestAf,
                        "P_CDFILLANC"   => $filialLanc,
                        "P_NRAUTOFORN"  => $nrAutoForn,
                        "P_CDPRODUTO"   => $cdproduto);
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function atualizaRelacionamentoAfNf($filial, $lancamento, $filialDestAf, $filialLanc, $nrAutoForn,
                                    $cdmarca, $cdproduto, $qtnfefetaf) {
        
        $sql = "UPDATE NFRELACAF SET QTNFEFETAF = QTNFEFETAF + :P_QTNFEFETAF
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF
                AND CDFILDEST  = :P_CDFILDEST
                AND CDFILLANC  = :P_CDFILLANC
                AND NRAUTOFORN = :P_NRAUTOFORN
                AND CDMARCA    = :P_CDMARCA
                AND CDPRODUTO  = :P_CDPRODUTO";
                
        $params = array("P_CDFILIAL"    => $filial,
                        "P_NRLANCTONF"  => $lancamento,
                        "P_CDFILDEST"   => $filialDestAf,
                        "P_CDFILLANC"   => $filialLanc,
                        "P_NRAUTOFORN"  => $nrAutoForn,
                        "P_CDMARCA"     => $cdmarca,
                        "P_CDPRODUTO"   => $cdproduto,
                        "P_QTNFEFETAF"  => $qtnfefetaf);
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }

    /**************************** Novo Imposto Item Temporario **********************************/
 /*   public function novoImpostoItemTemp(
        $filial, $lancamento, $seqItem, $imposto, $organizacao, $operador, $vrimpostoit,
        $vraliqitem, $vrbcalcimpoit = 0, $alteracaoUser = 'N', $nrseqitpro = 1, $vrcontimpoit = 0,
        $vrcontimpoit = 0, $vrredbcimpoit = 0, $vrpdedimpoit = 0, $idimpdesotrabit = 'N',
        $idmotdesotrabit = '9', $iddifaliqcuspti = 'N'
    ) {
        try {
            $data = DateUtil::getDataAtual();

            $itemImpostoTemp = new Trabitproimpos();

            $itemImpostoTemp->setCdfilimpoit($filial);
            $itemImpostoTemp->setNrlancimpoit($lancamento);
            $itemImpostoTemp->setNrseqimpoit($seqItem);
            $itemImpostoTemp->setCdimpostoit($imposto);
            $itemImpostoTemp->setNrseqitpro($nrseqitpro);

            $itemImpostoTemp->setNrorg($organizacao);
            $itemImpostoTemp->setCdoperinclusao($operador);
            $itemImpostoTemp->setDtinclusao($data);

            $itemImpostoTemp->setVrcontimpoit($vrcontimpoit);
            $itemImpostoTemp->setVrredbcimpoit($vrredbcimpoit);
            $itemImpostoTemp->setVrbcalcimpoit($vrbcalcimpoit);
            $itemImpostoTemp->setVrpdedimpoit($vrpdedimpoit);

            $itemImpostoTemp->setIdimpdesotrabit($idimpdesotrabit);
            $itemImpostoTemp->setIdmotdesotrabit($idmotdesotrabit);
            $itemImpostoTemp->setIddifaliqcuspti($iddifaliqcuspti);

            $itemImpostoTemp->setVrimposit($vrimpostoit);
            $itemImpostoTemp->setVraliqimpoit($vraliqitem);
            $itemImpostoTemp->setIdavisimpoit($alteracaoUser);
            $itemImpostoTemp->persist();

            return $itemImpostoTemp;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    */
 
    public function novoImpostoItemTemp(
        $filial, $lancamento, $seqItem, $imposto, $organizacao, $operador, $vrimpostoit,
        $vraliqitem, $vrbcalcimpoit = 0, $alteracaoUser = 'N', $nrseqitpro = 1, $vrcontimpoit = 0, $vrredbcimpoit = 0, $vrpdedimpoit = 0, $idimpdesotrabit = 'N',
        $idmotdesotrabit = '9', $iddifaliqcuspti = 'N', $vradicfecp = 0, $vradicaliqfecp = 0, $valorInfo = 'N', $cdstribimpoit = null) {
            
        $data = DateUtil::getDataAtual()->format("d/m/y H:i:s");
            
        $sql = "INSERT
                INTO TRABITPROIMPOS
                  (
                    CDFILIMPOIT,
                    NRLANCIMPOIT,
                    NRSEQIMPOIT,
                    CDIMPOSTOIT,
                    NRSEQITPRO,
                    VRCONTIMPOIT,
                    VRREDBCIMPOIT,
                    VRBCALCIMPOIT,
                    VRPDEDIMPOIT,
                    IDIMPDESOTRABIT,
                    IDMOTDESOTRABIT,
                    IDDIFALIQCUSPTI,
                    VRIMPOSIT,
                    VRALIQIMPOIT,
                    IDAVISIMPOIT,
                    VRADICFECP,
                    VRADICALIQFECP,
                    IDVALORINFO,
                    CDSTRIBIMPOIT,
                    NRORG,
                    CDOPERINCLUSAO,
                    DTINCLUSAO
                  )
                  VALUES
                  (
                      :P_CDFILIMPOIT,
                      :P_NRLANCIMPOIT,
                      :P_NRSEQIMPOIT,
                      :P_CDIMPOSTOIT,
                      :P_NRSEQITPRO,
                      :P_VRCONTIMPOIT,
                      :P_VRREDBCIMPOIT,
                      :P_VRBCALCIMPOIT,
                      :P_VRPDEDIMPOIT,
                      :P_IDIMPDESOTRABIT,
                      :P_IDMOTDESOTRABIT,
                      :P_IDDIFALIQCUSPTI,
                      :P_VRIMPOSIT,
                      :P_VRALIQIMPOIT,
                      :P_IDAVISIMPOIT,
                      :P_VRADICFECP,
                      :P_VRADICALIQFECP,
                      :P_IDVALORINFO,
                      :P_CDSTRIBIMPOIT,
                      :P_NRORG,
                      :P_CDOPERINCLUSAO,
                      :P_DTINCLUSAO
                  )";
        
        $params = array(
            "P_CDFILIMPOIT" => $filial,
            "P_NRLANCIMPOIT" => $lancamento,
            "P_NRSEQIMPOIT" => $seqItem,
            "P_CDIMPOSTOIT" => $imposto,
            "P_NRSEQITPRO" => $nrseqitpro,
            "P_VRCONTIMPOIT" => $vrcontimpoit,
            "P_VRREDBCIMPOIT" => $vrredbcimpoit,
            "P_VRBCALCIMPOIT" => $vrbcalcimpoit,
            "P_VRPDEDIMPOIT"  => $vrpdedimpoit,
            "P_IDIMPDESOTRABIT" => $idimpdesotrabit,
            "P_IDMOTDESOTRABIT" => $idmotdesotrabit,
            "P_IDDIFALIQCUSPTI" => $iddifaliqcuspti,
            "P_VRIMPOSIT" => $vrimpostoit,
            "P_VRALIQIMPOIT" => $vraliqitem,
            "P_IDAVISIMPOIT" => $alteracaoUser,
            "P_VRADICFECP" => $vradicfecp,
            "P_VRADICALIQFECP" => $vradicaliqfecp,
            "P_IDVALORINFO" => $valorInfo,
            "P_CDSTRIBIMPOIT" => '',
            "P_NRORG" => $organizacao,
            "P_CDOPERINCLUSAO" => $operador,
            "P_DTINCLUSAO" => $data
        );
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }

    /**************************** Editar Imposto Item Temporario ***************************************/
    
    public function editarImpostoItemTemp($filial, $lancamento, $seqItem, $imposto, $vrimpostoit, $vraliqitem,
        $alteracaoUser, $organizacao, $operador, $vrbcalcimpoit = 0, $idativo = 'S', $vrAdicFecp = 0, $vrAdicAliqFecp = 0, $valorinfo = 'N', $cdstribimpoit = null
        ) {
        
        $data = DateUtil::getDataAtual();    
        
        $sql = "UPDATE TRABITPROIMPOS
                SET VRIMPOSIT     = :P_VRIMPOSIT,
                  VRALIQIMPOIT    = :P_VRALIQIMPOIT,
                  VRBCALCIMPOIT   = :P_VRBCALCIMPOIT,
                  IDAVISIMPOIT    = :P_IDAVISIMPOIT,
                  VRADICFECP      = :P_VRADICFECP,
                  VRADICALIQFECP  = :P_VRADICALIQFECP,
                  IDVALORINFO     = :P_IDVALORINFO,
                  NRORGULTATU     = :P_NRORGULTATU,
                  DTULTATU        = :P_DTULTATU,
                  CDOPERULTATU    = :P_CDOPERULTATU
                WHERE CDFILIMPOIT = :P_CDFILIMPOIT
                AND NRLANCIMPOIT  = :P_NRLANCIMPOIT
                AND NRSEQIMPOIT   = :P_NRSEQIMPOIT
                AND CDIMPOSTOIT   = :P_CDIMPOSTOIT";
        
        if(gettype($data) == 'object') {
            $data = $data->format('d/m/Y');
        } 
        
        $params = array(
            "P_CDFILIMPOIT" => $filial,
            "P_NRLANCIMPOIT" => $lancamento,
            "P_NRSEQIMPOIT" => $seqItem,
            "P_CDIMPOSTOIT" => $imposto,
            "P_VRIMPOSIT" => $vrimpostoit,
            "P_VRALIQIMPOIT" => $vraliqitem,
            "P_VRBCALCIMPOIT"  => $vrbcalcimpoit,
            "P_IDAVISIMPOIT" => $alteracaoUser,
            "P_VRADICFECP" => $vrAdicFecp,
            "P_VRADICALIQFECP" => $vrAdicAliqFecp,
            "P_IDVALORINFO" => $valorinfo,
            "P_NRORGULTATU" => $organizacao,
            "P_DTULTATU"    => $data,
            "P_CDOPERULTATU" => $operador
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }

    /**************************** Novo Imposto Temporario ***************************************/
    public function novoImpostoTemp(
        $cdimpostot, $vrimpostot, $vrbcalimptot, $cdfilimptot, $nrlancimptot, $operador,
        $organizacao, $iddifaliqcusptn = 'N', $iderrostot = null, $idavisostot = null,
        $idativo = 'S'
    ) {
        try {
            $data = DateUtil::getDataAtual();
      
            $trabimposnftot = new Trabimposnftot();

            $trabimposnftot->setCdimpostot($cdimpostot);
            $trabimposnftot->setVrimpostot($vrimpostot);
            $trabimposnftot->setIderrostot($iderrostot);
            $trabimposnftot->setIdavisostot($idavisostot);
            $trabimposnftot->setCdfilimptot($cdfilimptot);
            $trabimposnftot->setNrlancimptot($nrlancimptot);
            $trabimposnftot->setVrbcalimptot($vrbcalimptot);
            $trabimposnftot->setIddifaliqcusptn($iddifaliqcusptn);
            $trabimposnftot->setNrorg($organizacao);
            $trabimposnftot->setNrorginclusao($organizacao);
            $trabimposnftot->setNrorgultatu($organizacao);
            $trabimposnftot->setDtinclusao($data);
            $trabimposnftot->setDtultatu($data);
            $trabimposnftot->setCdoperinclusao($operador);
            $trabimposnftot->setCdoperultatu($operador);
            //$trabimposnftot->setIdativo($idativo);
            $trabimposnftot->persist();

            return $trabimposnftot;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**************************** Editar Imposto Temporario ***************************************/
    public function editarImpostoTemp(
        Trabimposnftot $trabimposnftot, $vrimpostot, $vrbcalimptot,
        $iddifaliqcusptn, $iderrostot, $idavisostot, $operador, $organizacao, $idativo = 'S'
    ) {
        try {
            $data = DateUtil::getDataAtual();
         
            $trabimposnftot->setVrimpostot($vrimpostot);
            $trabimposnftot->setIderrostot($iderrostot);
            $trabimposnftot->setIdavisostot($idavisostot);
            $trabimposnftot->setVrbcalimptot($vrbcalimptot);
            $trabimposnftot->setIddifaliqcusptn($iddifaliqcusptn);
            $trabimposnftot->setNrorgultatu($organizacao);
            $trabimposnftot->setDtultatu($data);
            $trabimposnftot->setCdoperultatu($operador);
            //$trabimposnftot->setIdativo($idativo);
            $trabimposnftot->persist();

            return $trabimposnftot;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**************************** Cadastrar Conta Pagar ***************************************/
    public function novoContaPagar(
        $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdtpctpagar,
        $qtdiasvenpag, $dtorigvenpag, $dtatuavenpag, $vrorigpag,
        $vratuapag, $cdempresa, $organizacao, $operador, $idstatpag = 'A',
        $dscontapag = null, $cdmotiparc = null, $cdmotipror = null,
        $cdfilibpag = null, $nrlanbolpag = null, $cdclassfina = null,
        $cdfimovflpag = null, $dtmovfluxo = null, $idmovfluxo = null,
        $idexpocntpag = 'N', $vradiapagutl = null, $nrsequencialemp = null,
        $idativo = 'S', $iddscontanf = 'N', $idinterface = null,
        $tpinterface = null, $dtinterfaceori = null,
        $dtinterfacedest = null, $status = null
    ) {
        try {
            $data = DateUtil::getDataAtual();

            $contapagar = new Contapagar();

            $contapagar->setCdfilial($cdfilial);
            $contapagar->setNrlanctonf($nrlanctonf);
            $contapagar->setNrseqvenpag($nrseqvenpag);
            $contapagar->setNrparcelapag($nrparcelapag);
            $contapagar->setCdtpctpagar($cdtpctpagar);
            $contapagar->setQtdiasvenpag((is_null($qtdiasvenpag)? 0 : $qtdiasvenpag));
            $contapagar->setDtorigvenpag(self::formataData($dtorigvenpag));
            $contapagar->setDtatuavenpag(self::formataData($dtatuavenpag));
            $contapagar->setVrorigpag($vrorigpag);
            $contapagar->setCdempresa($cdempresa);

            $contapagar->setNrorg($organizacao);
            $contapagar->setDtinclusao($data);
            $contapagar->setDtultatu($data);
            $contapagar->setNrorginclusao($organizacao);
            $contapagar->setCdoperinclusao($operador);
            $contapagar->setNrorgultatu($organizacao);
            $contapagar->setCdoperultatu($operador);
            $contapagar->setIdativo($idativo);

            $contapagar->setCdmotiparc($cdmotiparc);
            $contapagar->setCdmotipror($cdmotipror);
            $contapagar->setVratuapag($vratuapag);
            $contapagar->setIdstatpag($idstatpag);
            $contapagar->setDscontapag($dscontapag);
            $contapagar->setCdfilibpag($cdfilibpag);
            $contapagar->setNrlanbolpag($nrlanbolpag);
            $contapagar->setCdclassfina($cdclassfina);
            $contapagar->setCdfimovflpag($cdfimovflpag);
            $contapagar->setDtmovfluxo($dtmovfluxo);
            $contapagar->setIdmovfluxo($idmovfluxo);
            $contapagar->setIdexpocntpag($idexpocntpag);
            $contapagar->setVradiapagutl($vradiapagutl);
            $contapagar->setNrsequencialemp($nrsequencialemp);

            //TODO Remover comentario quando utilizar a entidades correta
            //$iddscontanf     = $contapagar->setIddscontanf($iddscontanf);
            //TODO Remover quando homologar o script de criação
            //$idinterface     = $contapagar->setIdinterface($idinterface);
            //$tpinterface     = $contapagar->setTpinterface($tpinterface);
            //$dtinterfaceori  = $contapagar->setDtinterfaceori(self::formataData($dtinterfaceori));
            //$dtinterfacedest = $contapagar->setDtinterfacedest(self::formataData($dtinterfacedest));
            //$status          = $contapagar->setStatus($status);

            $contapagar->persist();

            return $contapagar;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**************************** Valida Nota Cadastrada **************************************/
    public function verificaNotaCadastrada($filial, $nrnotafisc, $serie, $nrinsjurnf) {
        try {
             $sql = <<<SQL
             SELECT COUNT(*)
               FROM NOTAFISCAL
              WHERE CDFILIRECE = :CDFILIRECE
                AND LPAD(NRNOTAFISC, 20, '0') = LPAD(:NRNOTAFISC, 20, '0')
                AND (:CDSERIEENTR IS NULL OR CDSERIEENTR = :CDSERIEENTR)
                AND (:NRINSJURNF IS NULL OR NRINSJURNF = :NRINSJURNF)
SQL;

            $params = array(
                'CDFILIRECE' => $filial,
                'CDSERIEENTR' => $serie,
                'NRINSJURNF' => $nrinsjurnf,
                'NRNOTAFISC' => $nrnotafisc
            );
            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            return $query->fetchColumn();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaEditNotaCadastrada($filial, $serie, $nrinsjurnf, $nrnotafisc){
        $sql = "SELECT NRLANCTONF
                    FROM NOTAFISCAL
                WHERE CDFILIRECE = :CDFILIRECE
                    AND LPAD(NRNOTAFISC, 20, '0') = LPAD(:NRNOTAFISC, 20, '0')
                    AND (:CDSERIEENTR IS NULL OR CDSERIEENTR = :CDSERIEENTR)
                    AND (:NRINSJURNF IS NULL OR NRINSJURNF = :NRINSJURNF)";
                    
        $params = array(
                        "CDFILIRECE"    => $filial,
                        "NRNOTAFISC"    => $nrnotafisc,
                        "NRINSJURNF"    => $nrinsjurnf,
                        "CDSERIEENTR"    => $serie
                        
                        );
                        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    /**************************** Alteração Nota Fiscal Eletronica ***************************************/
    public function updateNrnotafiscalele($notaFiscal, $nrnotafiscalele) {
        try {
             $notaFiscal->setNrnotafiscalele($nrnotafiscalele);

             $notaFiscal->persist();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaDeleteProdfornxml($cdforneced, $cdprodutoxml) {
        try {
            $prodfornxml = Prodfornxml::findOneByCdfornecedAndCdprodutoxml($cdforneced, $cdprodutoxml);

            if (!!$prodfornxml) {
                $this->deleteProdfornxml($prodfornxml);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaRateio($cdfilial,$nrLancto){
        $sql = "SELECT * 
                FROM RATEIONF
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array(
            "P_NRLANCTONF"        => $nrLancto,
            "P_CDFILIAL"          => $cdfilial
        );
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function buscaDeleteProdfornxmlFilial($cdforneced, $cdproduto, $filial, $CDPRODUTOXML_BACKUP_ROW) {
        $sql = "DELETE FROM PRODFORNXMLFIL
                WHERE CDFORNECED = :P_CDFORNECED
                AND CDPRODUTOXML   = :P_CDPRODUTOXML
                AND CDFILIAL     = :P_CDFILIAL
                AND CDPRODUTO    = :P_CDPRODUTO"
                ;
                
        $params = array("P_CDFORNECED"  => $cdforneced,
                        "P_CDPRODUTOXML"   => $cdproduto,
                        "P_CDFILIAL"    => $filial,
                        "P_CDPRODUTO"    => $CDPRODUTOXML_BACKUP_ROW );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);    
    }

    public function deleteProdfornxml(Prodfornxml $prodfornxml) {
        try {
             $prodfornxml->delete();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function novoProdfornxml(
        $cdproduto, $cdforneced, $cdprodutoxml, $nmprodutoxml,
        $organizacao, $operador, $idativo = 'S'
    ) {
        try {
            $data = DateUtil::getDataAtual();

            $prodfornxml = new Prodfornxml();

            $prodfornxml->setCdproduto($cdproduto);
            $prodfornxml->setCdforneced($cdforneced);
            $prodfornxml->setCdprodutoxml($cdprodutoxml);
            $prodfornxml->setNmprodutoxml($nmprodutoxml);
            $prodfornxml->setDtinclusao($data);
            $prodfornxml->setDtultatu($data);
            $prodfornxml->setNrorg($organizacao);
            $prodfornxml->setNrorginclusao($organizacao);
            $prodfornxml->setNrorgultatu($organizacao);
            $prodfornxml->setCdoperinclusao($operador);
            $prodfornxml->setCdoperultatu($operador);
            // $prodfornxml->setIdativo($idativo);

            $prodfornxml->persist();

            return $prodfornxml;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function novoProdfornxmlFilial(
        $cdproduto, $cdforneced, $cdprodutoxml, $nmprodutoxml, $cdfilial,
        $organizacao, $operador, $idativo = 'S'
    ) {
        try {
            
            $sql = "INSERT INTO PRODFORNXMLFIL (CDPRODUTO, CDFORNECED, CDFILIAL, CDPRODUTOXML, NMPRODUTOXML, CDOPERINCLUSAO, CDOPERULTATU)
               VALUES (:CDPRODUTO, :CDFORNECED, :CDFILIAL, :CDPRODUTOXML, :NMPRODUTOXML, :CDOPERADOR, :CDOPERADOR)";
            
            $params = array("CDPRODUTO"     => $cdproduto,
                            "CDFORNECED"    => $cdforneced,
                            "CDFILIAL"      => $cdfilial,
                            "CDPRODUTOXML"  => $cdprodutoxml,
                            "NMPRODUTOXML"  => $nmprodutoxml,
                            "CDOPERADOR"    => $operador);
                            
            $std = $this->entityManager->getConnection()->prepare($sql);
            return $std->execute($params);      
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getNFOrigemDevolucao($params) {
        $sql = "SELECT CDFILIAL, NRLANCTONF, IDTPIJURNF,
                       NRINSJURNF, CDSERIE, NRNOTAFISC,
                       DTENTRSAID, DTEMISSAO, NVL(VRNOTAFISC, 0) AS VRNOTAFISC,
                       TO_CHAR(DTEMISSAO) AS DATAEMISSAO,
                         TO_CHAR(FLOOR(VRNOTAFISC))||','||SUBSTR(TO_CHAR(VRNOTAFISC-FLOOR(VRNOTAFISC))||'00',2,2) VALORNF
                FROM NOTAFISCAL N
                WHERE (IDTPIJURNF = NVL(:IDTPIJURNF,IDTPIJURNF))    AND
                      (NRINSJURNF = NVL(:NRINSJURNF,NRINSJURNF))    AND
                      (NVL(CDFILIRECE,'T') = NVL(:CDFILIRECE,'T'))  AND
                      (CDFILIEMIT   = :CDFILIORIG)                    AND
                      (IDENTRSAIDOP = 'S')                            AND
                      (IDSTATUS IN ('A','E'))                         AND
                        ((LPAD(NRNOTAFISC,20,'0') = LPAD(:NRNOTAFISC,20,'0')) OR (:NRNOTAFISC = 'T'))   AND
                      (CDSERIE    = NVL(:CDSERIE,CDSERIE)) AND
                      (NVL(IDSISTEMA, 'DFS') <> 'REC')                AND
                        ((INSTR(:CDTIPOOPSA, CDTIPOOPER) > 0) OR (:CDTIPOOPSA IS NULL AND :CDTIPOOPEN IS NULL)) AND
                      (N.IDNFDEVOLVIDA = 'N') AND
                      ((:RELACDOORI = 'S' AND
                         N.CDFILIAL||N.NRLANCTONF NOT IN (SELECT I.CDFILIALIT||I.NRLANCTONFIT
                                                      FROM ITEMNOTAFISC I, NOTAFISCAL NF
                                                      WHERE I.CDFILIALIT   = NF.CDFILIAL   AND
                                                            I.NRLANCTONFIT = NF.NRLANCTONF AND
                                                            NF.CDFILIEMIT = :CDFILIORIG      AND
                                                           (NVL(NF.CDFILIRECE,'T') = NVL(:CDFILIRECE,'T'))  AND
                                                            NF.IDTPIJURNF = NVL(:IDTPIJURNF,N.IDTPIJURNF) AND
                                                            NF.NRINSJURNF = NVL(:NRINSJURNF,N.NRINSJURNF))) OR (NVL(:RELACDOORI,'N') <> 'S'))
                UNION ALL
                SELECT
                   A.CDFILIAL, A.NRLANCTONF, A.IDTPIJURNF,
                   A.NRINSJURNF, A.CDSERIEENTR AS CDSERIE, NRNOTAFISC,
                   DTENTRSAID, DTEMISSAO, NVL(VRNOTAFISC,0) AS VRNOTAFISC,
                   TO_CHAR(DTEMISSAO) AS DATAEMISSAO,
                     TO_CHAR(FLOOR(VRNOTAFISC))||','||SUBSTR(TO_CHAR(VRNOTAFISC-FLOOR(VRNOTAFISC))||'00',2,2) VALORNF
                FROM NOTAFISCAL A
                WHERE
                   (A.IDTPIJURNF = NVL(:IDTPIJURNF,A.IDTPIJURNF))      AND
                   (A.NRINSJURNF = NVL(:NRINSJURNF,A.NRINSJURNF))      AND
                   (NVL(A.CDFILIEMIT,'T') = NVL(:CDFILIRECE,'T'))      AND
                     ((LPAD(NRNOTAFISC,20,'0') = LPAD(:NRNOTAFISC,20,'0')) OR (:NRNOTAFISC = 'T'))   AND
                   (A.CDSERIEENTR    = NVL(:CDSERIE,A.CDSERIEENTR)) AND
                   (A.CDFILIRECE = :CDFILIORIG)                          AND
                   (CDTIPOOPER <> :CDTIPOOPER)                         AND
                   (A.IDENTRSAIDOP = 'E')                                AND
                   ((A.IDSISTEMA IN ('DFE')) OR (A.IDSISTEMA IS NULL))   AND
                     ((INSTR(:CDTIPOOPEN, A.CDTIPOOPER) > 0) OR (:CDTIPOOPSA IS NULL AND :CDTIPOOPEN IS NULL)) AND
                   (A.IDNFDEVOLVIDA = 'N') AND
                   ((:RELACDOORI = 'S' AND
                      A.CDFILIAL||A.NRLANCTONF NOT IN (SELECT I.CDFILIALIT||I.NRLANCTONFIT
                                                   FROM ITEMNOTAFISC I, NOTAFISCAL NF
                                                   WHERE I.CDFILIALIT   = NF.CDFILIAL   AND
                                                         I.NRLANCTONFIT = NF.NRLANCTONF AND
                                                         NF.CDFILIRECE = :CDFILIORIG      AND
                                                        (NVL(NF.CDFILIEMIT,'T') = NVL(:CDFILIRECE,'T'))      AND
                                                         NF.IDTPIJURNF = NVL(:IDTPIJURNF,A.IDTPIJURNF) AND
                                                         NF.NRINSJURNF = NVL(:NRINSJURNF,A.NRINSJURNF))) OR (NVL(:RELACDOORI,'N') <> 'S'))
                ORDER BY
                      CDFILIAL, NRLANCTONF";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }
    
    public function atualizaIntgNotaFiscal($idStatusNfe, $idImportada, $idUpdataData, $cdFiliRece, $cdFilial, $nrLanctonf, $nrAcessoNfe) {
        $sql = "UPDATE INTG_NOTAFISCAL
                 SET IDSTATUSNFE = :P_IDSTATUSNFE,
                     IDIMPORTADA = :P_IDIMPORTADA,
                     CDFILIRECE  = DECODE(:P_IDUPDATDATA, 'S', :P_CDFILIRECE, CDFILIRECE),
              
                       DTUTILIZAXML = DECODE(:P_IDUPDATDATA, 'S', SYSDATE, NULL),
                     CDFILIAL     = :P_CDFILIAL,
                     NRLANCTONF   = :P_NRLANCTONF
               WHERE NRACESSONFE = :P_NRACESSONFE";
               
        $params = array("P_IDSTATUSNFE" => $idStatusNfe,
                        "P_IDIMPORTADA" => $idImportada,
                        "P_IDUPDATDATA" => $idUpdataData,
                        "P_CDFILIRECE"  => $cdFiliRece,
                        "P_CDFILIAL"    => $cdFilial,
                        "P_NRLANCTONF"  => $nrLanctonf,
                        "P_NRACESSONFE" => $nrAcessoNfe);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);        
    }
    
    public function insereHistoricoXML($nrAcessoNfe, $nrSeqHistor, $dtHistorico, $idTipoOper, $idStatus, $nrProcoloNFe, $dsOperacao, $cdFilial, $nrLanctonf, $cdOperador, $nrOrg) {
        $sql = "INSERT INTO HISTXMLNFE(NRACESSONFE, NRSEQHISTOR, DTHISTORICO, IDTIPOOPER,
                               IDSTATUS, NRPROTOCOLONFE, DSOPERACAO, CDFILIAL,
                             NRLANCTONF, CDOPERADOR, NRORG)
               VALUES (:NRACESSONFE, :NRSEQHISTOR, :DTHISTORICO, :IDTIPOOPER,
                :IDSTATUS, :NRPROTOCOLONFE, :DSOPERACAO, :CDFILIAL,
                :NRLANCTONF, :CDOPERADOR, :NRORG)";
              
              
        $params = array("NRACESSONFE"		=> $nrAcessoNfe,
                        "NRSEQHISTOR"		=> $nrSeqHistor, 
                        "DTHISTORICO"		=> $dtHistorico,
                        "IDTIPOOPER"		=> $idTipoOper,
                        "IDSTATUS"			=> $idStatus,
                        "NRPROTOCOLONFE"	=> $nrProcoloNFe,
                        "DSOPERACAO"		=> $dsOperacao,
                        "CDFILIAL"			=> $cdFilial,
                        "NRLANCTONF"		=> $nrLanctonf,
                        "CDOPERADOR"		=> $cdOperador,
                        "NRORG"				=> $nrOrg);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getSequencialHistoricoXML($nrAcessoNfe) {
        $sql = "SELECT MAX(NVL(NRSEQHISTOR,0)) AS NRSEQHISTOR
                FROM HISTXMLNFE
               WHERE NRACESSONFE = :P_NRACESSONFE";
         
        $params = array("P_NRACESSONFE"  => $nrAcessoNfe);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
    }
    
    public function getDadosNfIntg($cdFilial, $nrLanctonf) {
        $sql = "SELECT NRACESSONFE
                FROM NOTAFISCAL
                WHERE CDFILIAL       = :P_CDFILIAL
                AND NRLANCTONF       = :P_NRLANCTONF
                AND IDNFGERADAVIAXML = 'S'
                AND IDENTRSAIDOP     = 'E'";
        
        $params = array("P_CDFILIAL"    => $cdFilial,
                        "P_NRLANCTONF"  => $nrLanctonf);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch(); 
    }
    
    public function verificaNotaXml($cdFilial, $nrLanctonf) {
        $sql = "SELECT IDSTATUSNFE
                FROM INTG_NOTAFISCAL
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array("P_CDFILIAL"    => $cdFilial,
                        "P_NRLANCTONF"  => $nrLanctonf);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
    }
    
    public function getIdbloqxmlinsdif() {
        $sql = "SELECT IDBLOQXMLINSDIF FROM PARAMGERAL";
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
        $aux = $std->fetch();
        if($aux){
            return $aux["IDBLOQXMLINSDIF"];
        } else {
            return null;
        }
        
    }
    
    public function getNrinsjurest($nracessonfe) {
        $sql = "SELECT NRINSJUREMPR FROM INTG_NOTAFISCAL WHERE NRACESSONFE = :P_NRACESSONFE";
        
        $params = array(
            "P_NRACESSONFE" => $nracessonfe
        );
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $aux = $std->fetch();
        if($aux){
            return $aux["NRINSJUREMPR"];
        } else {
            return null;
        }
        
    }
    
    public function aprovaNfXmlAutomatica() {
        $sql = "SELECT IDPERAPROVNFXML FROM PARAMGERAL";
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
        return $std->fetchAll(); 
    }
    
     public function limpaCapa($cdFilial, $nrLanctonf) {
        $sql = "UPDATE NOTAFISCAL SET NRSEQCAPADOC = '' WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF";
        
        $params = array("P_CDFILIAL"    => $cdFilial,
                        "P_NRLANCTONF"  => $nrLanctonf);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }

    
    public function getDefaultFilter($cdFilial) {
         $sql = "SELECT E.CDEMPRESA,
                  E.NMEMPRESA,
                  F.NMFILIAL,
                  F.CDFILIAL AS CDFILIRECE,
                  F.CDFILIAL AS CDFILIAL,
                  PF.DTPROCESSA,
                  F.NRORG
                FROM EMPRESA E,
                  FILIAL F,
                  PARAMFILIAL PF
                WHERE F.CDEMPRESA = E.CDEMPRESA
                AND F.CDFILIAL    = :P_CDFILIAL
                AND F.CDFILIAL    = PF.CDFILIAL";
        
        $params = array(
            "P_CDFILIAL"        => $cdFilial
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getDefaultNovaNota($cdFilial, $cdOperador) {
        $sql = "SELECT FO.CDOPERADOR,
                  F.NMFILIAL,
                  F.CDEMPRESA,
                  E.NMEMPRESA,
                  F.CDINSCESTA,
                  F.CDINSCESTA AS IEDEST,
                  F.CDINSCESTA AS IEREM,
                  F.CDINSCMUNI,
                  F.NRORG,
                  F.IDTPIJURFILI,
                  F.NRINSJURFILI,
                  PF.IDUTBLOQOPAFPF,
                  PF.IDBLOQCMPSAFFIL,
                  NVL(PF.NRTAMNUMNFUTDFE, 9) AS NRTAMNUMNFUTDFE,
                  PF.IDCANCEXTMPNFE,
                  PF.DSPARFILFIS,
                  DECODE(PF.NRMAXDIAANTDTAF, NULL, 'T', 'N')                                                                                        AS DTMINVAZIA,
                  DECODE(PF.NRMAXDIAANTDTAF, NULL, NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)), NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)) - PF.NRMAXDIAANTDTAF) AS DTPROCINI,
                  DECODE(PF.NRMAXDIAPOSDTAF, NULL, 'T', 'N')                                                                                        AS DTMAXVAZIA,
                  DECODE(PF.NRMAXDIAPOSDTAF, NULL, NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)), NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)) + PF.NRMAXDIAPOSDTAF) AS DTPROCFIN,
                  PG.IDBLOQITNFAPUR,
                  PG.IDOBRCHAVACES,
                  PG.IDPELANIMPXML,
                  DECODE(SUBSTR(PF.DSPARFILFIS, 2, 1), 'S', 'S', PG.IDBLOQCMPSAF) AS IDBLOQAF,
                  DECODE(PF.IDUTBLOQOPAFPF, 'N', PG.IDUTBLOQOPAFPG, 'S')          AS IDUTBLOQOPAF,
                  PG.IDUTBLOQOPAFPG,
                  PG.IDBLOQCMPSAF,
                  NVL(PG.IDGERNFIGUALXML, 'N') AS IDGERNFIGUALXML,
                  EN.CDPAIS,
                  EN.SGESTADO AS SGESTADODEFAULT,
                  M.NMMUNICIPIO AS NMMUNICIPIODEFAULT,
                  EN.DSENDEFILI,
                  EN.NMBAIRFILI,
                  EN.NRCEPFILI,
                  EN.NRTELEFILI,
                  NVL(PI.IDPERIMPSPRAF, 'N') AS IDPERIMPSPRAF,
                  F.IDFUNCAOFILI,
                  PG.IDBLOQALTPROXML,
                  PF.DTPROCESSA,
                  PF.CDFILIAL AS CDFILIRECE
                FROM FILIOPER FO
                JOIN FILIAL F
                ON FO.CDFILIAL    = F.CDFILIAL
                AND FO.CDOPERADOR = :P_CDOPERADOR
                LEFT JOIN EMPRESA E
                ON E.CDEMPRESA = F.CDEMPRESA
                AND E.NRORG    = F.NRORG
                INNER JOIN PARAMFILIAL PF
                ON PF.CDFILIAL = F.CDFILIAL
                INNER JOIN PARAMGERAL PG
                ON PG.NRORG = F.NRORG
                JOIN ENDEFILI EN
                ON F.CDFILIAL       = EN.CDFILIAL
                AND F.NRORG         = EN.NRORG
                AND EN.IDTPENDEFILI = 'P'
                JOIN MUNICIPIO M
                ON E.CDPAIS       = M.CDPAIS
                AND E.SGESTADO    = M.SGESTADO
                AND E.CDMUNICIPIO = M.CDMUNICIPIO
                LEFT JOIN PARAMINTEGRA PI
                ON CDPARAMINT = '1'
                WHERE F.CDFILIAL = :P_CDFILIAL
                ORDER BY F.CDFILIAL";
                
         $params = array(
            "P_CDFILIAL"        => $cdFilial,
            "P_CDOPERADOR"      => $cdOperador
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getIDATUESTTPOP($identrsaidop, $cdtipooper) {
        $sql = "SELECT IDATUESTTPOP
                  
                FROM TIPOOPERAC
                
                WHERE IDENTRSAIDOP = :P_IDENTRSAIDOP
                AND CDTIPOOPER = :P_CDTIPOOPER";
                
         $params = array(
            "P_IDENTRSAIDOP"        => $identrsaidop,
            "P_CDTIPOOPER"          => $cdtipooper
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function fornecedorVerificaChave($cnpjForn) {
        $sql = "SELECT IDVERINSFORCHNFE FROM PARAMFORNE WHERE CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_CDFORNECED"        => $cnpjForn
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["IDVERINSFORCHNFE"];
    }
    
    public function existeItemSemRelacionamento($chaveAcesso, $fornecedor, $filial) {
        
        try {
            $relacionamentoFilial = $this->utilizaRelacionamentoPorFilial();
            if(isset($relacionamentoFilial["IDUTRELXMLFIL"])) {
                $relacionamentoFilial = ($relacionamentoFilial["IDUTRELXMLFIL"] == 'S');
            }
    
        }
        catch(\Exception $e) {
            $relacionamentoFilial = false;
        }
        
                        
        $params = array(
            "NRACESSONFE"        => $chaveAcesso,
            "CDFORNECED"         => $fornecedor
        );
        
        if(!$relacionamentoFilial) {
            $sql = "SELECT I.CDPRODUTO AS CDPRODUTOINTG,
                  I.NMPRODUTO AS NMPRODUTOINTG
                FROM INTG_ITEMPRODUTO I,
                  PRODFORNXML P,
                  PRODUTO PR
                WHERE I.NRACESSONFE   = :NRACESSONFE
                AND P.CDFORNECED(+)   = :CDFORNECED
                AND P.CDPRODUTOXML(+) = I.CDPRODUTO
                AND PR.CDPRODUTO (+)  = P.CDPRODUTO
                AND P.CDPRODUTOXML IS NULL
                ORDER BY TO_NUMBER(I.NRSEQITPROD)";
        }
        else {
            $sql = "SELECT I.CDPRODUTO AS CDPRODUTOINTG,
                  I.NMPRODUTO AS NMPRODUTOINTG
                FROM INTG_ITEMPRODUTO I,
                  PRODFORNXML P,
                  PRODFORNXMLFIL PF, 
                  PRODUTO PR
                WHERE I.NRACESSONFE   = :NRACESSONFE
                AND P.CDFORNECED(+)   = :CDFORNECED
                AND PF.CDFILIAL(+)    = :CDFILIAL 
                AND PF.CDFORNECED(+)  = :CDFORNECED
                AND PF.CDPRODUTOXML(+) = I.CDPRODUTO
                AND P.CDPRODUTOXML(+) = I.CDPRODUTO
                AND PR.CDPRODUTO (+)  = P.CDPRODUTO
                AND P.CDPRODUTOXML IS NULL
                AND PF.CDPRODUTOXML IS NULL
                ORDER BY TO_NUMBER(I.NRSEQITPROD)";   
        
            $params["CDFILIAL"] = $filial;
        }

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        return count($std->fetchAll()) != 0 ? true : false;
    }
    
    public function verificaAfObrigatoria() {
        $sql = "SELECT NVL(IDPERIMPSPRAF, 'N') AS IDPERIMPSPRAF FROM PARAMINTEGRA";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
        $obrigaAf = ($std->fetch()["IDPERIMPSPRAF"] == 'S') ? true : false;
    }
    
    public function getItensAf($cdFilial, $nrAutoForn) {
        $sql = "SELECT DISTINCT CDPRODUTO FROM ITEMAUTF
                WHERE CDFILDEST = :P_CDFILIAL
                AND NRAUTOFORN IN $nrAutoForn";
                
        $params = array(
            "P_CDFILIAL"        => $cdFilial
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getItensNotaOrigem($notaorig, $filiOrig, $nrlanctonf, $cdfilial) {
        $sql = "SELECT B.NRORG,
                  SUBSTR(:CDFILIAL,1,4)    AS CDFILIAL,
                  SUBSTR(:NRLANCTONF,1,10) AS NRLANCTONF,
                  C.NRSEQITPROD,
                  C.CDPRODUTO,
                  B.NMPRODUTO,
                  B.SGUNIDADE,
                  B.CDARVPROD,
                  NVL(A.CDPRODEMIS, C.CDPRODEMIS)    AS CDPRODEMIS,
                  NVL(A.IDCONFORITEM,C.IDCONFORITEM) AS IDCONFORITEM,
                  NVL(A.DSPRODEMIS, C.DSPRODEMIS)    AS DSPRODEMIS,
                  NVL(A.CDFISOPEITEM,C.CDFISOPEITEM) AS CDFISOPEITEM,
                  NVL(A.IDATUVRESTOQ,C.IDATUVRESTOQ) AS IDATUVRESTOQ,
                  NVL(A.CDSUFXCFOPPR,C.CDSUFXCFOPPR) AS CDSUFXCFOPPR,
                  NVL(A.CDSITUTRIBPR,C.CDSITUTRIBPR) AS CDSITUTRIBPR,
                  NVL(A.IDAVISOSITEM,C.IDAVISOSITEM) AS IDAVISOSITEM,
                  NVL(A.VRUNITITEM,C.VRUNITITEM)     AS VRUNITITEM,
                  NVL(A.VRDESCITEM,C.VRDESCORIG)     AS VRDESCITEM,
                  NVL(SUM(A.VRTOTAITEM),0.0)         AS VRTOTAITEM,
                  NVL(SUM(A.QTITEMPROD),0.0)         AS QTITEMPROD,
                  NVL(SUM(A.VRCONTITEM),0.0)         AS VRCONTITEM,
                  NVL(SUM(A.VRBRUTITEM),0.0)         AS VRBRUTITEM,
                  NVL(A.VRDESPERITEM,C.VRDESPERITEM) AS VRDESPERITEM,
                  NVL(A.CDFILIMOVI,C.CDFILIMOVI)     AS CDFILIMOVI,
                  NVL(A.CDALMOXARIFE,C.CDALMOXARIFE) AS CDALMOXARIFE,
                  NVL(A.CDLOCALESTOQ,C.CDLOCALESTOQ) AS CDLOCALESTOQ,
                  NVL(A.NRLOTEESTQ,C.NRLOTEESTQ)     AS NRLOTEESTQ,
                  NVL(A.NRSUBLOTE,C.NRSUBLOTE)       AS NRSUBLOTE,
                  B.CDPRODESTO,
                  (QTITEMORIG),
                  (VRCONTIORIG),
                  (VRDESCORIG),
                  (VRBRUTIORIG),
                  ''            AS CDFILDEST,
                  ''            AS NRAUTOFORN,
                  ''            AS CDMARCA,
                  TO_DATE(NULL) AS DTPROGENTAF,
                  B.VRFATOCONV,
                  SUBSTR(B.CDARVPROD
                  ||' ',1,20) AS CDPRODANT,
                  ' '         AS IDOPER,
                  ' '         AS IDDELETE,
                  C.QTITRELACNFSUBS,
                  NVL(C.IDITEMORIGEM, 'N') AS IDITEMORIGEM,
                  A.IDORIGEMPROD
                FROM ITEMPRODUTO A,
                  PRODUTO B,
                  (SELECT LPAD(ROWNUM,4,0) AS NRSEQITPROD,
                    CDPRODUTO,
                    QTITEMORIG,
                    IDCONFORITEM,
                    CDFISOPEITEM,
                    IDATUVRESTOQ,
                    CDSUFXCFOPPR,
                    CDSITUTRIBPR,
                    IDAVISOSITEM,
                    VRUNITITEM,
                    CDFILIMOVI,
                    CDALMOXARIFE,
                    CDLOCALESTOQ,
                    NRLOTEESTQ,
                    NRSUBLOTE,
                    VRCONTIORIG,
                    VRDESCORIG,
                    VRDESPERITEM,
                    VRBRUTIORIG,
                    QTITRELACNFSUBS,
                    IDITEMORIGEM,
                    IDORIGEMPROD,
                    CDPRODEMIS,
                    DSPRODEMIS
                  FROM
                    (SELECT
                      /*+ INDEX (P PK_ITEMPRODUTO) */
                      CDPRODUTO,
                      SUM(QTITEMPROD) QTITEMORIG,
                      IDCONFORITEM,
                      CDFISOPEITEM,
                      IDATUVRESTOQ,
                      CDSUFXCFOPPR,
                      CDSITUTRIBPR,
                      IDAVISOSITEM,
                      VRUNITITEM,
                      CDFILIMOVI,
                      CDALMOXARIFE,
                      CDLOCALESTOQ,
                      NRLOTEESTQ,
                      NRSUBLOTE,
                      SUM(VRCONTITEM) AS VRCONTIORIG,
                      SUM(VRDESCITEM) AS VRDESCORIG,
                      SUM(VRDESPERITEM) VRDESPERITEM,
                      SUM(VRBRUTITEM)             AS VRBRUTIORIG,
                      SUM(NVL(QTITRELACNFSUBS,0)) AS QTITRELACNFSUBS,
                      'S'                         AS IDITEMORIGEM,
                      IDORIGEMPROD,
                      CDPRODEMIS,
                      DSPRODEMIS
                    FROM ITEMPRODUTO I
                    WHERE I.CDFILIAL = :CDFILIORIG
                    AND I.NRLANCTONF IN (:NRLANCTONFORIG)
                    GROUP BY CDPRODUTO,
                      IDCONFORITEM,
                      CDFISOPEITEM,
                      IDATUVRESTOQ,
                      CDSUFXCFOPPR,
                      CDSITUTRIBPR,
                      IDAVISOSITEM,
                      VRUNITITEM,
                      CDFILIMOVI,
                      CDALMOXARIFE,
                      CDLOCALESTOQ,
                      NRLOTEESTQ,
                      NRSUBLOTE,
                      IDORIGEMPROD,
                      CDPRODEMIS,
                      DSPRODEMIS
                    )
                  ) C
                WHERE (A.CDFILIAL(+)           = :CDFILIAL)
                AND (A.NRLANCTONF(+)           = :NRLANCTONF)
                AND (A.CDPRODUTO(+)            = C.CDPRODUTO)
                AND (A.VRUNITITEM(+)           = C.VRUNITITEM)
                AND (NVL(A.NRLOTEESTQ(+), ' ') = NVL(C.NRLOTEESTQ, ' '))
                AND (NVL(A.NRSUBLOTE(+), ' ')  = NVL(C.NRSUBLOTE, ' '))
                AND (B.CDPRODUTO               = C.CDPRODUTO)
                GROUP BY B.NRORG,
                  SUBSTR(:CDFILIAL,1,4),
                  SUBSTR(:NRLANCTONF,1,10),
                  C.NRSEQITPROD,
                  C.CDPRODUTO,
                  B.NMPRODUTO,
                  B.SGUNIDADE,
                  B.CDARVPROD,
                  NVL(A.CDPRODEMIS,C.CDPRODEMIS),
                  NVL(A.IDCONFORITEM,C.IDCONFORITEM),
                  NVL(A.DSPRODEMIS,C.DSPRODEMIS),
                  NVL(A.CDFISOPEITEM,C.CDFISOPEITEM),
                  NVL(A.IDATUVRESTOQ,C.IDATUVRESTOQ),
                  NVL(A.CDSUFXCFOPPR,C.CDSUFXCFOPPR),
                  NVL(A.CDSITUTRIBPR,C.CDSITUTRIBPR),
                  NVL(A.IDAVISOSITEM,C.IDAVISOSITEM),
                  NVL(A.VRUNITITEM,C.VRUNITITEM),
                  NVL(A.VRDESCITEM,C.VRDESCORIG),
                  NVL(A.VRDESPERITEM,C.VRDESPERITEM),
                  NVL(A.CDFILIMOVI,C.CDFILIMOVI),
                  NVL(A.CDALMOXARIFE,C.CDALMOXARIFE),
                  NVL(A.CDLOCALESTOQ,C.CDLOCALESTOQ),
                  NVL(A.NRLOTEESTQ,C.NRLOTEESTQ),
                  NVL(A.NRSUBLOTE,C.NRSUBLOTE),
                  B.CDPRODESTO,
                  (QTITEMORIG),
                  (VRCONTIORIG),
                  (VRDESCORIG),
                  (VRBRUTIORIG),
                  '',
                  '',
                  '',
                  TO_DATE(NULL),
                  B.VRFATOCONV,
                  SUBSTR(B.CDARVPROD
                  ||' ',1,20),
                  ' ',
                  ' ',
                  C.QTITRELACNFSUBS,
                  NVL(C.IDITEMORIGEM, 'N'),
                  A.IDORIGEMPROD
                ORDER BY 1,2,3";
                
        $params = array(
            "NRLANCTONFORIG"    => $notaorig,
            "CDFILIORIG"        => $filiOrig,
            "NRLANCTONF"        => $nrlanctonf,
            "CDFILIAL"          => $cdfilial
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getFirstItem($filial, $lancamento) {
        $sql = "SELECT NRSEQITPROD, CDARVPROD, VRTOTAITEM
                FROM ITEMPRODUTO IP, PRODUTO P 
                WHERE CDFILIAL  = :P_CDFILIAL
                AND IP.CDPRODUTO = P.CDPRODUTO
                AND NRLANCTONF = :P_NRLANCTONF 
                AND NRSEQITPROD = 
                  (SELECT MIN(NRSEQITPROD)
                  FROM ITEMPRODUTO
                  WHERE CDFILIAL = :P_CDFILIAL
                  AND NRLANCTONF = :P_NRLANCTONF
                  )";
                  
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getIdValidInsXML($cdtipooper, $cdfilial) {
        $sql = "SELECT IDVALIDINSXML
                FROM PARAMBLOQNFAFIL
                WHERE CDTIPOOPER = :P_CDTIPOOPER
                    AND CDFILIAL   = :P_CDFILIAL";
                  
        $params = array(
            "P_CDTIPOOPER"   =>  $cdtipooper,
            "P_CDFILIAL"     =>  $cdfilial
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch() ? $std->fetch()["IDVALIDINSXML"] : "N";
    }
    
    public function utilizaRelacionamentoPorFilial() {
            
        $sql = "SELECT IDUTRELXMLFIL FROM PARAMINTEGRA PI";
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
        return $std->fetch();
    }
    
    public function getInfoNota($filial, $lancamento) {
         $sql = "SELECT IDATIVO,
                        DTINCLUSAO,
                        DTULTATU,
                        NRORGINCLUSAO,
                        CDOPERINCLUSAO,
                        NRORGULTATU,
                        CDOPERULTATU
                FROM NOTAFISCAL
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function verificaRegistros($cdfilial, $lancamento) {
        //$connection = $this->entityManager->getConnection();
        //$connection->beginTransaction();
        $sql = "SELECT
                    COUNT(*) AS QTDREGISTRO
                FROM
                    INFOCOMPRANFE
                WHERE
                    CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF";
                    
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $lancamento
        );
                    
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["QTDREGISTRO"];
        
    }
    public function atualizaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg){
        $qtdRegistros = $this->verificaRegistros($filial, $lancamento);
        if($qtdRegistros > 0){
            $informacoes = $this->getInfoNota($filial, $lancamento);
            $this->alteraInfoCompra($informacoes,$filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg);
        }
    }
    
    public function alteraInfoCompra($informacoes,$filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg){
        
        $idAtivo = $informacoes[0]['IDATIVO'];
        $dtInclusao = $informacoes[0]['DTINCLUSAO'];
        $dtUltatu = $informacoes[0]['DTULTATU'];
        $nrorgInclusao = $informacoes[0]['NRORGINCLUSAO'];
        $cdoperinclusao = $informacoes[0]['CDOPERINCLUSAO'];
        $nrorgUltatu = $informacoes[0]['NRORGULTATU'];
        $cdoperUltatu = $informacoes[0]['CDOPERULTATU'];
        
       
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
         $sql = "UPDATE INFOCOMPRANFE 
                    SET 
                    DSNOTAEMPENHO = :DSNOTAEMPENHO,
                    CDPEDIDO = :CDPEDIDO,
                    CDCONTRATO = :CDCONTRATO,
                    IDATIVO = :IDATIVO,
                    DTINCLUSAO = :DTINCLUSAO,
                    DTULTATU = :DTULTATU,
                    NRORGINCLUSAO = :NRORGINCLUSAO,
                    CDOPERINCLUSAO = :CDOPERINCLUSAO,
                    NRORGULTATU = :NRORGULTATU,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORG = :NRORG
                    WHERE  CDFILIAL = :CDFILIAL
                        AND NRLANCTONF = :NRLANCTONF";
        
        $params = array(
            "CDFILIAL"        =>  $filial,
            "NRLANCTONF"      =>  $lancamento,
            "DSNOTAEMPENHO"   =>  $notaEmp,
            "CDPEDIDO"        =>  $pedido,
            "CDCONTRATO"      =>  $contrato,
            "IDATIVO"         =>  $idAtivo,
            "DTINCLUSAO"      =>  $dtInclusao,
            "DTULTATU"        =>  $dtUltatu,
            "NRORGINCLUSAO"   =>  $nrorgInclusao,
            "CDOPERINCLUSAO"  =>  $cdoperinclusao,
            "NRORGULTATU"     =>  $nrorgUltatu,
            "CDOPERULTATU"    =>  $cdoperUltatu,
            "NRORG"           =>  $nrorg 
        );
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $connection->commit();
        
        //var_dump($lancamento,$filial,$notaEmp,$pedido,$contrato);
        //die;
    }
    
    public function salvaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg){
        
        $informacoes = $this->getInfoNota($filial, $lancamento);
        //$this->verificaRegistros($filial, $lancamento);
        $idAtivo = $informacoes[0]['IDATIVO'];
        $dtInclusao = $informacoes[0]['DTINCLUSAO'];
        $dtUltatu = $informacoes[0]['DTULTATU'];
        $nrorgInclusao = $informacoes[0]['NRORGINCLUSAO'];
        $cdoperinclusao = $informacoes[0]['CDOPERINCLUSAO'];
        $nrorgUltatu = $informacoes[0]['NRORGULTATU'];
        $cdoperUltatu = $informacoes[0]['CDOPERULTATU'];
        
        //$connection = $this->entityManager->getConnection();
        //$connection->beginTransaction();
        
         $sql = "INSERT INTO INFOCOMPRANFE (
                CDFILIAL,
                NRLANCTONF,
                DSNOTAEMPENHO,
                CDPEDIDO,
                CDCONTRATO,
                IDATIVO,
                DTINCLUSAO,
                DTULTATU,
                NRORGINCLUSAO,
                CDOPERINCLUSAO,
                NRORGULTATU,
                CDOPERULTATU,
                NRORG
            ) VALUES (
                :P_CDFILIAL,
                :P_NRLANCTONF,
                :P_DSNOTAEMPENHO,
                :P_CDPEDIDO,
                :P_CDCONTRATO,
                :P_IDATIVO,
                :P_DTINCLUSAO,
                :P_DTULTATU,
                :P_NRORGINCLUSAO,
                :P_CDOPERINCLUSAO,
                :P_NRORGULTATU,
                :P_CDOPERULTATU,
                :P_NRORG
            )";
            
        $params = array(
            "P_CDFILIAL"        =>  $filial,
            "P_NRLANCTONF"      =>  $lancamento,
            "P_DSNOTAEMPENHO"   =>  $notaEmp,
            "P_CDPEDIDO"        =>  $pedido,
            "P_CDCONTRATO"      =>  $contrato,
            "P_IDATIVO"         =>  $idAtivo,
            "P_DTINCLUSAO"      =>  $dtInclusao,
            "P_DTULTATU"        =>  $dtUltatu,
            "P_NRORGINCLUSAO"   =>  $nrorgInclusao,
            "P_CDOPERINCLUSAO"  =>  $cdoperinclusao,
            "P_NRORGULTATU"     =>  $nrorgUltatu,
            "P_CDOPERULTATU"    =>  $cdoperUltatu,
            "P_NRORG"           =>  $nrorg 
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        //$connection->commit();
        
        //var_dump($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg,$idAtivo,$idAtivo,$dtInclusao,$dtUltatu,$cdoperinclusao
        //,$cdoperinclusao,$nrorgUltatu,$cdoperUltatu);
        //die;
        
    }
    
    public function verificaNotaSendoUtilizada($filial, $lancamento) {
        $sql = "SELECT CDFILIAL
                FROM NOTAFISCAL
                WHERE CDFILIAL    = :P_CDFILIAL
                AND NRLANCTONF    = :P_NRLANCTONF  FOR UPDATE NOWAIT";
                
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        
        try {
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            $result = $std->fetch();
            return false;
        }
        catch(\Exception $e) {
            return true;
        }
    }
    
    public function getVencimentos($filial, $lancamento) {
        $sql = "SELECT *
                FROM CONTAPAGAR
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
        
    }
    
    public function validaModeloDocumento($idEntrsaidop, $cdttipooper, $cdmoddocfisc) {
        $sql = "SELECT
                    MT.CDMODDOCFISC,
                    M.DSMODDOCFISC,
                    MT.IDENTRSAIDOP,
                    MT.CDTIPOOPER,
                    M.IDMODDOCNFE,
                    MT.NRORG
                FROM
                    MODDOCFISC       M,
                    MODDOCFISCTPOP   MT
                WHERE
                    M.CDMODDOCFISC = MT.CDMODDOCFISC
                    AND M.NRORG = MT.NRORG
                    AND MT.CDTIPOOPER   = :P_CDTIPOOPER
                    AND MT.IDENTRSAIDOP = :P_IDENTRSAIDOP
                    AND MT.CDMODDOCFISC = :P_CDMODDOCFISC
                ORDER BY
                    CDMODDOCFISC";
                
        $params = array(
            "P_IDENTRSAIDOP"      => $idEntrsaidop,
            "P_CDTIPOOPER"        => $cdttipooper,
            "P_CDMODDOCFISC"      => $cdmoddocfisc
        );
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function validaBloqueioGeral() {
        $sql = "SELECT P.IDBLOQCMPSAF FROM PARAMGERAL P
                WHERE P.CDPARGERAL = :P_CDPARGERAL";
                
        $params = array("P_CDPARGERAL" => '0001');
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();
        return $result ? $result[0]["IDBLOQCMPSAF"] : null;
    }
    
    public function validaBloqueioFilialForn($cdfilial) {
        $sql = "SELECT P.IDBLOQCMPSAFFIL FROM PARAMFILIAL P
                    WHERE P.CDFILIAL = :P_CDFILIAL";
                
        $params = array("P_CDFILIAL" => $cdfilial);
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();
        return $result ? $result[0]["IDBLOQCMPSAFFIL"] : null;
    }
    
    public function validaBloqueioFilial($cdfilial) {
        $sql = "SELECT P.DSPARFILFIS FROM PARAMFILIAL P 
                    WHERE P.CDFILIAL = :P_CDFILIAL";
                
        $params = array("P_CDFILIAL" => $cdfilial);
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();
        $paramResult = $result ? substr($result[0]["DSPARFILFIS"], 1, 1) : null;
        return $paramResult;
    }
    
    
    public function validaBloqueioFornecedor($cdforneced) {
        $sql = "SELECT F.IDBLOQCMPSAFFOR FROM FORNECEDOR F
                    WHERE F.CDFORNECED = :P_CDFORNECED";
                
        $params = array("P_CDFORNECED" => $cdforneced);
        

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetchAll();
        return $result ? $result[0]["IDBLOQCMPSAFFOR"] : null;
    }
    
    public function getAnexoNfe($cdfilial, $nrlanctonf, $conditions = null){
        $sql = "SELECT
                    A.DSLINKANEXO                 
             FROM
                ANEXONFE A
             WHERE 1=1
                 AND A.CDFILIAL = :P_CDFILIAL
                 AND A.NRLANCTONF = :P_NRLANCTONF
                 $conditions";
        
        $params = array(
            "P_CDFILIAL"    => $cdfilial,
            "P_NRLANCTONF"  => $nrlanctonf
        );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getItemProduto($params) {
        $sql = "SELECT
                 P.CDARVPROD,
                 IT.CDPRODUTO,
                 IT.NRSEQITPROD,
                 IT.VRTOTAITEM
             FROM
                 ITEMPRODUTO   IT,
                 PRODUTO       P
             WHERE
                 IT.CDFILIAL = :P_CDFILIAL
                 AND IT.NRLANCTONF = :P_NRLANCTONF
                 AND IT.NRSEQITPROD = :P_NRSEQITPROD
                 AND P.CDPRODUTO = IT.CDPRODUTO";
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getFuncFili($cdfilial){
        $sql = "SELECT NVL(F.IDFUNCAOFILI, 'N') AS IDFUNCAOFILI 
                FROM FILIAL F
                    WHERE F.CDFILIAL = :P_CDFILIAL";
        
        $params = array(
            "P_CDFILIAL" => $cdfilial    
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
        return $result["IDFUNCAOFILI"];
    }
    
    public function getProdIgXml($cdforneced){
        $sql = "SELECT NVL(P.IDUTPRIGXML, 'N') AS IDUTPRIGXML 
                FROM PARAMFORNE P 
                WHERE P.CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_CDFORNECED" => $cdforneced    
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
        return $result["IDUTPRIGXML"];
    }
    
    public function getFilialAF($cdfilial) {
        $sql = "SELECT
                    DECODE(PF.IDUTBLOQOPAFPF, 'N', PG.IDUTBLOQOPAFPG, 'S') AS IDUTBLOQOPAF
                    FROM PARAMFILIAL PF, PARAMGERAL PG
                    WHERE PF.CDFILIAL = :P_CDFILIAL";
                    
        $params = array(
            "P_CDFILIAL" => $cdfilial    
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()['IDUTBLOQOPAF'];
    }
    
    public function verificaParamProdutosAF($cdfilial) {
        $sql = "SELECT
                    P.IDBLOQNFSAF
                    FROM
                    PARAMCMPPROD P
                    WHERE
                    (P.CDFILIAL = :P_CDFILIAL) AND
                    (P.IDBLOQNFSAF = 'N')";
                    
        $params = array(
            "P_CDFILIAL" => $cdfilial    
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function verificaExigeAf($cdfilial, $cdtipooper, $idutbloqopaf) {
        $sql = "SELECT CDTIPOOPER, NMTIPOOPER, IDENTRSAIDOP, IDUTILOPER,
                    NVL(IDORIGDESTOP,'F') AS IDORIGDESTOP, IDGERAVENC, IDUTILVENC,
                    IDUTILEMISNF, IDATUESTTPOP, IDOPERDEVONF, DSOPERNFORIG,
                    'N' AS IDEXIGEAF, IDINFCPFCNPJNF, CDFORNECEDPADR, DSOPERNFAVISTA, IDREARATMANNF
                    FROM TIPOOPERAC
                    WHERE IDENTRSAIDOP = 'E'
                    AND IDUTILOPER <> 'N'
                    AND CDTIPOOPER = NVL(:P_CDTIPOOPER, CDTIPOOPER)
                    AND :P_IDUTBLOQOPAF = 'N'
                    
                    UNION ALL
                    
                    SELECT T.CDTIPOOPER, T.NMTIPOOPER, T.IDENTRSAIDOP, T.IDUTILOPER,
                    NVL(T.IDORIGDESTOP,'F') AS IDORIGDESTOP, T.IDGERAVENC, T.IDUTILVENC,
                    T.IDUTILEMISNF, T.IDATUESTTPOP, T.IDOPERDEVONF, T.DSOPERNFORIG,
                    'N' as IDEXIGEAF, IDINFCPFCNPJNF, CDFORNECEDPADR, DSOPERNFAVISTA, IDREARATMANNF
                    FROM TIPOOPERAC T
                    WHERE T.CDTIPOOPER = NVL(:P_CDTIPOOPER, T.CDTIPOOPER) AND
                    T.IDENTRSAIDOP = 'E' AND
                    :P_IDUTBLOQOPAF = 'S' AND
                    T.CDTIPOOPER NOT IN (SELECT P.CDTIPOOPER
                    FROM PARAMBLOQNFAFIL P
                    WHERE P.CDFILIAL = :P_CDFILIAL AND
                    P.IDENTRSAIDOP = 'E' AND
                    P.CDTIPOOPER = NVL(:P_CDTIPOOPER, P.CDTIPOOPER))
                    
                    UNION ALL
                    
                    SELECT T.CDTIPOOPER, T.NMTIPOOPER, T.IDENTRSAIDOP, T.IDUTILOPER,
                    NVL(T.IDORIGDESTOP,'F') AS IDORIGDESTOP, T.IDGERAVENC, T.IDUTILVENC,
                    T.IDUTILEMISNF, T.IDATUESTTPOP, T.IDOPERDEVONF, T.DSOPERNFORIG,
                    P.IDEXIGEAF, IDINFCPFCNPJNF, CDFORNECEDPADR, DSOPERNFAVISTA, IDREARATMANNF
                    FROM PARAMBLOQNFAFIL P, TIPOOPERAC T
                    WHERE P.CDFILIAL = :P_CDFILIAL
                    AND P.IDENTRSAIDOP = 'E'
                    AND P.CDTIPOOPER = NVL(:P_CDTIPOOPER, P.CDTIPOOPER)
                    AND P.IDLIBERATPOPFIL = 'S'
                    AND :P_IDUTBLOQOPAF = 'S'
                    AND P.CDTIPOOPER = T.CDTIPOOPER
                    AND P.IDENTRSAIDOP = T.IDENTRSAIDOP
                    ORDER BY CDTIPOOPER";
                    
        $params = array(
            "P_CDFILIAL"        => $cdfilial,
            "P_CDTIPOOPER"      => $cdtipooper,
            "P_IDUTBLOQOPAF"    => $idutbloqopaf
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()['IDEXIGEAF'];
    }
    
    public function insereItemNotaFisc($cdfilial, $nrlanctonf, $cdfilialit, $nrlanctonfit, $nrorg) {
        $sql = "SELECT MAX(NRSEQUITNOTA) AS NRSEQUITNOTA FROM ITEMNOTAFISC WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF";
        
        $params = array(
            "P_CDFILIAL"        => $cdfilial,
            "P_NRLANCTONF"      => $nrlanctonf
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $nrseqitem = $std->fetch()["NRSEQUITNOTA"];
        
        if(!$nrseqitem) {
            $nrseqitem = 0;
        }
        else {
            $nrseqitem += 1;
        }
        $nrseqitem = str_pad($nrseqitem, 3, '0', STR_PAD_LEFT);
        
        $sql = "INSERT INTO ITEMNOTAFISC (
                CDFILIAL,
                NRLANCTONF,
                NRSEQUITNOTA,
                CDFILIALIT,
                NRLANCTONFIT,
                NRORG
            ) VALUES (
                :P_CDFILIAL,
                :P_NRLANCTONF,
                :P_NRSEQUITNOTA,
                :P_CDFILIALIT,
                :P_NRLANCTONFIT,
                :P_NRORG
            )";
            
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf,
            "P_NRSEQUITNOTA" => $nrseqitem,
            "P_CDFILIALIT" => $cdfilialit,
            "P_NRLANCTONFIT" => $nrlanctonfit,
            "P_NRORG" => $nrorg
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function deleteItemnotafisc($cdfilial, $nrlanctonf) {
        $sql = "DELETE FROM ITEMNOTAFISC WHERE CDFILIALIT = :P_CDFILIAL AND NRLANCTONFIT = :P_NRLANCTONF";
        
        $params = array(
            "P_CDFILIAL"        => $cdfilial,
            "P_NRLANCTONF"      => $nrlanctonf
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function deleteAnexoNF($cdfilial, $nrlanctonf){
        $sql = "DELETE FROM ANEXONFE WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF";
        
        $params = array(
            "P_CDFILIAL"        => $cdfilial,
            "P_NRLANCTONF"      => $nrlanctonf
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getValorIndicePorDataEmissao($cdindice, $cdtpindice, $idpervalor, $dtrefind, $nrorg) {
        //cdtpindice 1: Diário   2: Mensal   3: Anual
        $sql = "SELECT
                    CDINDICE,
                    VRINDICE,
                    DTREFIND
                FROM
                    MOVINDICE
                WHERE
                    CDINDICE = :CDINDICE
                    AND NRORG = :NRORG
                    AND ( ( :CDTPINDICE = '1'
                            AND DTREFIND = TO_DATE(:DTREFIND, 'DD/MM/YYYY') )
                          OR ( :CDTPINDICE = '2'
                               AND EXTRACT(MONTH FROM DTREFIND) = EXTRACT(MONTH FROM TO_DATE(:DTREFIND, 'DD/MM/YYYY')) 
                               AND EXTRACT(YEAR FROM DTREFIND) = EXTRACT(YEAR FROM TO_DATE(:DTREFIND, 'DD/MM/YYYY')) )
                          OR ( :CDTPINDICE = '3'
                               AND EXTRACT(YEAR FROM DTREFIND) = EXTRACT(YEAR FROM TO_DATE(:DTREFIND, 'DD/MM/YYYY')) ) )
                    AND CDTPINDICE = :CDTPINDICE
                    AND IDPERVALOR = :IDPERVALOR";
        $params = array(
                    "CDINDICE" => $cdindice,
                    "DTREFIND" => $dtrefind,
                    "NRORG"    => $nrorg,
                    "CDTPINDICE" => $cdtpindice,
                    "IDPERVALOR" => $idpervalor
                    );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["VRINDICE"];
    }
    
    public function salvaAnexo($nome, $operador, $organizacao, $nrlanctonf, $cdfilial, $nrseqanexo){
       $sql = "INSERT INTO 
               ANEXONFE
               (CDFILIAL, NRLANCTONF, DSLINKANEXO, NRSEQANEXO, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) 
               VALUES
               ( :P_CDFILIAL, :P_NRLANCTONF, :P_DSLINKANEXO, :P_NRSEQANEXO, :P_NRORG, 'S', SYSDATE, SYSDATE, :P_NRORG, :P_CDOPER, :P_NRORG, :P_CDOPER)";
               
       $params = array("P_NRORG" => $organizacao,
                       "P_CDFILIAL" => $cdfilial,
                       "P_NRLANCTONF" => $nrlanctonf,
                       "P_DSLINKANEXO" => $nome,
                       "P_NRSEQANEXO" => $nrseqanexo,
                       "P_CDOPER" => $operador);
                       
       $std = $this->entityManager->getConnection()->prepare($sql);
       $std->execute($params);
    }
    
    public function validaParamProduto($row){
        $sql = "SELECT
                    P.IDBLOQNFSAF, PR.CDARVPROD
                FROM
                    PARAMCMPPROD  P,
                    PRODUTO       PR
                WHERE
                    ( P.CDFILIAL = :P_CDFILIAL )
                    AND ( INSTR(:P_CDARVPROD, PR.CDARVPROD) = 1 )
                    AND ( P.CDPRODUTO = PR.CDPRODUTO )
                ORDER BY
                    PR.CDARVPROD DESC";
         
        $params = array("P_CDFILIAL"    => $row['CDFILIAL'],
                        "P_CDARVPROD"   => $row['CDARVPROD']);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
        return $result ? $result : []; 
    }
    
    public function getSequencialAnexonfe($cdfilial, $nrlanctonf) {
        $sql = "SELECT NVL(MAX(NVL(NRSEQANEXO, 0)), 0) + 1 AS NRSEQANEXO
                FROM ANEXONFE
               WHERE CDFILIAL = :P_CDFILIAL
               AND NRLANCTONF = :P_NRLANCTONF";
         
        $params = array("P_CDFILIAL"    => $cdfilial,
                        "P_NRLANCTONF"  => $nrlanctonf);
                        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch(); 
    }
   
   
   /**
    *  Operação 11
    */
    
    public function verificaNracessoOpOnze($cdfilial, $nrlanctonf, $nrorg){
        $sql =  "select IDNFGERADAVIAXML 
                        from notafiscal 
                    where 
                        nrlanctonf = :nrlanctonf
                        and cdfilial = :cdfilial
                        and nrorg = :nrorg";
                    
        $params = array("cdfilial"    => $cdfilial,
    	                "nrlanctonf"  => $nrlanctonf,
                        "nrorg"       => $nrorg
                    );
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();             
    }
    
   public function getItensEntradaSemNota($cdfilial, $nrlanctonf, $cdfilialit, $nrlanctonfit, $nrorg) {
       $sql = "SELECT
                    IP.CDPRODUTO,
                    IP.CDPRODEMIS,
                    IP.DSPRODEMIS,
                    IP.DTPROGENTAF,
                    IP.CDALMOXARIFE,
                    IP.CDLOCALESTOQ,
                    IP.NRLOTEESTQ,
                    IP.NRSUBLOTE,
                    IP.CDFILIMOVI,
                    IP.CDOPERADOR,
                    P.CDARVPROD,
                    IP.NRSEQITPROD,
                    SUM(IP.QTITEMPROD) AS QTITEMPROD,
                    IP.VRUNITITEM AS VRUNITITEM,
                    SUM(IP.VRDESCITEM) AS VRDESCITEM,
                    SUM(IP.VRTOTAITEM) AS VRTOTAITEM
                FROM
                    ITEMNOTAFISC   IF,
                    ITEMPRODUTO    IP,
                    PRODUTO        P
                WHERE
                    ( IF.CDFILIAL = :P_CDFILIAL )
                    AND ( IF.NRLANCTONF = :P_NRLANCTONF )
                    AND (INSTR(:P_NRLANCTONFIT, '_'||IF.NRLANCTONFIT||'_') > 0)
                    AND (IF.CDFILIALIT = :P_CDFILIALIT)
                    AND ( IP.CDFILIAL = IF.CDFILIALIT )
                    AND ( IP.NRLANCTONF = IF.NRLANCTONFIT )
                    AND (P.CDPRODUTO = IP.CDPRODUTO)
                    AND (P.NRORG = :P_NRORG)
                GROUP BY
                    IP.CDPRODUTO,
                    IP.CDPRODEMIS,
                    IP.DSPRODEMIS,
                    IP.DTPROGENTAF,
                    IP.VRUNITITEM,
                    IP.CDALMOXARIFE,
                    IP.CDLOCALESTOQ,
                    IP.NRLOTEESTQ,
                    IP.NRSUBLOTE,
                    IP.CDFILIMOVI,
                    IP.CDOPERADOR,
                    P.CDARVPROD,
                    IP.NRSEQITPROD";
                
		$params = array("P_CDFILIAL"    => $cdfilial,
		                "P_NRLANCTONF"  => $nrlanctonf,
		                "P_CDFILIALIT" => $cdfilialit,
                        "P_NRLANCTONFIT" => $nrlanctonfit,
                        "P_NRORG" => $nrorg
                        );
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
   }
   
   public function insereItemProduto($cdfilial, $nrlanctonf, $cdprodemis, $vrunititem, $idconforitem, $dsprodemis, $vrdescitem, $nrseqitprod,
                                    $idatuvrestoq, $vrtotaitem, $cdproduto, $qtitemprod, $vrcontitem, $vrbrutitem, $cdfilimovi,
                                    $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdoperador) {
                                        
        $sql = "INSERT INTO ITEMPRODUTO (
            CDFILIAL,
            CDPRODEMIS,
            VRUNITITEM,
            IDCONFORITEM,
            NRLANCTONF,
            DSPRODEMIS,
            VRDESCITEM,
            NRSEQITPROD,
            IDATUVRESTOQ,
            VRTOTAITEM,
            CDPRODUTO,
            QTITEMPROD,
            VRCONTITEM,
            VRBRUTITEM,
            CDALMOXARIFE,
            CDLOCALESTOQ,
            NRLOTEESTQ,
            NRSUBLOTE,
            CDFILIMOVI,
            CDOPERADOR
        ) VALUES (
            :CDFILIAL,
            :CDPRODEMIS,
            :VRUNITITEM,
            :IDCONFORITEM,
            :NRLANCTONF,
            :DSPRODEMIS,
            :VRDESCITEM,
            :NRSEQITPROD,
            :IDATUVRESTOQ,
            :VRTOTAITEM,
            :CDPRODUTO,
            :QTITEMPROD,
            :VRCONTITEM,
            :VRBRUTITEM,
            :CDALMOXARIFE,
            :CDLOCALESTOQ,
            :NRLOTEESTQ,
            :NRSUBLOTE,
            :CDFILIMOVI,
            :CDOPERADOR
        )";
        
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDPRODEMIS" => $cdprodemis,
            "VRUNITITEM" => $vrunititem,
            "IDCONFORITEM" => $idconforitem,
            "NRLANCTONF" => $nrlanctonf,
            "DSPRODEMIS" => $dsprodemis,
            "VRDESCITEM" => $vrdescitem,
            "NRSEQITPROD" => $nrseqitprod,
            "IDATUVRESTOQ" => $idatuvrestoq,
            "VRTOTAITEM" => $vrtotaitem,
            "CDPRODUTO" => $cdproduto,
            "QTITEMPROD" => $qtitemprod,
            "VRCONTITEM" => $vrcontitem,
            "VRBRUTITEM" => $vrbrutitem,
            "CDALMOXARIFE" => $cdalmoxarife,
            "CDLOCALESTOQ" => $cdlocalestoq,
            "NRLOTEESTQ" => $nrloteestq,
            "NRSUBLOTE" => $nrsublote,
            "CDFILIMOVI" => $cdfilimovi,
            "CDOPERADOR" => $cdoperador
        );
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);

   }
   
   public function getVrindiceItmAf($cdfildest, $nrautoforn, $cdproduto, $cdmarca) {
        $sql = "SELECT
                    VRINDICEITMAF
                FROM
                    ITEMAUTF
                WHERE
                    CDFILDEST = :P_CDFILDEST
                    AND NRAUTOFORN = :P_NRAUTOFORN
                    AND CDPRODUTO = :P_CDPRODUTO
                    AND CDMARCA   = :P_CDMARCA";
                    
        $params = array(
            "P_CDFILDEST"     => $cdfildest,
            "P_NRAUTOFORN"    => $nrautoforn,
            "P_CDPRODUTO"     => $cdproduto, 
            "P_CDMARCA"       => $cdmarca
        );
                    
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["VRINDICEITMAF"];
    }
   
   public function consultaNotaUtilizaAfIndice($cdfilial, $nrlanctonf) {
        $sql = "SELECT
                    COUNT(*) AS UTILIZA_INDICE
                FROM
                    NFRELACAF   N,
                    AUTOFORNE    A
                WHERE
                    CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF
                    AND A.CDFILDEST = N.CDFILDEST
                    AND A.NRAUTOFORN = N.NRAUTOFORN
                    AND A.CDINDICEAF IS NOT NULL";
                    
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
                    
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["UTILIZA_INDICE"];
    }
    
    public function getProdutoAfImportadaIndice($cdfilial, $nrlanctonf) {
        $sql = "SELECT 
                    A.CDFILLANC,
                    N.NRLANCTONF,
                    N.CDFILIAL,
                    A.CDTPINDICEAF AS CDTPINDICE,
                    A.IDPERVALORAF AS IDPERVALOR,
                    I.CDPRODUTO,
                    IA.VRINDICEITMAF AS VRPREUNIAF,
                    IA.VRDESCITMAF,
                    A.CDINDICEAF AS CDINDICE,
                    A.NRAUTOFORN,
                    IA.VRPERDESAF,
                    IA.QTEFETENTRAF,
                    IA.QTPROGENTAF,
                    IA.IDBAIXITMAF
                FROM
                    ITEMPRODUTO I,
                    ITEMAUTF  IA,
                    AUTOFORNE  A,
                    NOTAFISCAL N
                WHERE
                    IA.NRAUTOFORN = A.NRAUTOFORN
                    AND IA.CDPRODUTO = I.CDPRODUTO
                    AND A.NRAUTOFORN = I.NRAUTOFORN
                    AND A.CDFILDEST = N.CDFILIRECE
                    AND I.NRLANCTONF = N.NRLANCTONF
                    AND I.CDFILIAL = N.CDFILIAL
                    AND N.CDFILIAL = :P_CDFILIAL
                    AND N.NRLANCTONF = :P_NRLANCTONF";
                    
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
                    
    }
    
    public function getDadosVenc($cdtipooper, $identrsaid, $cdForneced) {
        $sql = "SELECT CDTIPOOPER, NMTIPOOPER, IDGERAVENC, IDUTILVENC, CDFORNECED, IDTPFORPGTFO FROM TIPOOPERAC, FORNECEDOR F WHERE CDTIPOOPER = :P_CDTIPOOPER AND IDENTRSAIDOP = :P_IDENTRSAIDOP AND CDFORNECED = :P_CDFORNECED";
        
        $params = array(
            "P_CDTIPOOPER"      => $cdtipooper,
            "P_IDENTRSAIDOP"    => $identrsaid, 
            "P_CDFORNECED"      => $cdForneced  
        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getRetencoes($cdfilial, $nrlanctonf) {
        $sql = "SELECT NVL(SUM(VRIMPOSTOT), 0) AS VRRETENCOES FROM TRABIMPOSNFTOT IT, IMPOSTO I WHERE IT.CDIMPOSTOT = I.CDIMPOSTO AND IT.CDFILIMPTOT  = :P_CDFILIAL AND IT.NRLANCIMPTOT = :P_NRLANCTONF AND I.IDTRATIMPO    = 'R' AND I.IDDESCRFONTE  = 'R'";
        
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["VRRETENCOES"];
    }
    
    public function subtVrBruto($cdfilial, $nrlanctonf) {
        $sql = "SELECT NVL(SUM(VRBRUTITEM), 0) AS VRBRUTITEM
                FROM NOTAFISCAL N, TPOPERCFOP T, ITEMPRODUTO I
               WHERE T.IDENTRSAIDOP = N.IDENTRSAIDOP
                 AND T.CDTIPOOPER   = N.CDTIPOOPER
                 AND T.IDCOMPOEVENC = 'N'
                 AND T.CDFISCOPER   = I.CDFISOPEITEM
                 AND N.CDFILIAL   = :P_CDFILIAL
                 AND N.NRLANCTONF = :P_NRLANCTONF";
                 
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["VRBRUTITEM"];
    }
    
    public function getContAf($cdfilial, $nrlanctonf) {
        $sql = "SELECT DISTINCT
                    CDFILDEST,
                    NRAUTOFORN,
                    CDFILLANC
                FROM
                    NFRELACAF
                WHERE
                    ( CDFILIAL = :P_CDFILIAL )
                    AND ( NRLANCTONF = :P_NRLANCTONF )
                ORDER BY
                    NRAUTOFORN";
                 
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getAfVenc($cdfildest, $cdfillanc, $nrautoforn) {
        $sql = "SELECT
                    AUTOFORNE.CDFILLANC
                FROM
                    AUTOFORNE
                WHERE
                    AUTOFORNE.CDFILDEST = :P_CDFILDEST
                    AND AUTOFORNE.CDFILLANC = :P_CDFILLANC
                    AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN
                    AND AUTOFORNE.IDTPFORPGTAF = 'V'
                    AND NOT EXISTS (
                        SELECT
                            PRAZOPGTAF.CDFILLANC
                        FROM
                            PRAZOPGTAF
                        WHERE
                            AUTOFORNE.CDFILDEST = PRAZOPGTAF.CDFILDEST
                            AND AUTOFORNE.CDFILLANC = PRAZOPGTAF.CDFILLANC
                            AND AUTOFORNE.NRAUTOFORN = PRAZOPGTAF.NRAUTOFORN
                    )
                UNION ALL
                SELECT
                    AF.CDFILLANC
                FROM
                    AUTOFORNE    AF
                    INNER JOIN FORNECEDOR   F ON AF.CDFORNECED = F.CDFORNECED
                WHERE
                    AF.CDFILDEST = :P_CDFILDEST
                    AND AF.CDFILLANC = :P_CDFILLANC
                    AND AF.NRAUTOFORN = :P_NRAUTOFORN
                    AND F.IDTPFORPGTFO = 'V'
                        AND NOT EXISTS ( SELECT
                                     PRAZOPGTFORN.CDFORNECED
                                 FROM
                                     PRAZOPGTFORN
                                 WHERE
                                     PRAZOPGTFORN.CDFORNECED = F.CDFORNECED";
                 
        $params = array(
            "P_CDFILDEST" => $cdfildest,
            "P_CDFILLANC" => $cdfillanc,
            "P_NRAUTOFORN" => $nrautoforn
        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getMaxSeqItem($cdfilial, $nrlanctonf) {
        $sql = "SELECT MAX(NRSEQITPROD) AS NRSEQITPROD FROM ITEMPRODUTO WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF";
        
        $params = array(
            "P_CDFILIAL"        => $cdfilial,
            "P_NRLANCTONF"      => $nrlanctonf
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $nrseqitem = $std->fetch()["NRSEQITPROD"];
    }
    
    public function existeItemProduto($cdfilial, $nrlanctonf, $cdprodemis, $vrunititem, $idconforitem, $dsprodemis, $vrdescitem,
                                    $idatuvrestoq,  $cdproduto, $qtitemprod, $vrcontitem, $vrbrutitem, $cdfilimovi,
                                    $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdoperador) {
                                        
        $sql = "SELECT
                    CDFILIAL,
                    CDPRODEMIS,
                    VRUNITITEM,
                    NRSEQITPROD,
                    IDCONFORITEM,
                    NRLANCTONF,
                    DSPRODEMIS,
                    VRDESCITEM,
                    IDATUVRESTOQ,
                    VRTOTAITEM,
                    CDPRODUTO,
                    QTITEMPROD,
                    VRCONTITEM,
                    VRBRUTITEM,
                    CDALMOXARIFE,
                    CDLOCALESTOQ,
                    NRLOTEESTQ,
                    NRSUBLOTE,
                    CDFILIMOVI,
                    CDOPERADOR
                FROM
                    ITEMPRODUTO
                WHERE
                    CDFILIAL = :CDFILIAL
                    AND CDPRODEMIS = :CDPRODEMIS
                    AND VRUNITITEM = :VRUNITITEM
                    AND IDCONFORITEM = :IDCONFORITEM
                    AND NRLANCTONF = :NRLANCTONF
                    AND DSPRODEMIS = :DSPRODEMIS
                    AND VRDESCITEM = :VRDESCITEM
                    AND IDATUVRESTOQ = :IDATUVRESTOQ
                    AND CDPRODUTO = :CDPRODUTO
                    AND VRCONTITEM = :VRCONTITEM
                    AND CDALMOXARIFE = :CDALMOXARIFE
                    AND CDLOCALESTOQ = :CDLOCALESTOQ
                    AND NRLOTEESTQ = :NRLOTEESTQ
                    AND NRSUBLOTE = :NRSUBLOTE
                    AND CDFILIMOVI = :CDFILIMOVI
                    AND CDOPERADOR = :CDOPERADOR";
        
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDPRODEMIS" => $cdprodemis,
            "VRUNITITEM" => $vrunititem,
            "IDCONFORITEM" => $idconforitem,
            "NRLANCTONF" => $nrlanctonf,
            "DSPRODEMIS" => $dsprodemis,
            "VRDESCITEM" => $vrdescitem,
            "IDATUVRESTOQ" => $idatuvrestoq,
            "CDPRODUTO" => $cdproduto,
            "VRCONTITEM" => $vrcontitem,
            "CDALMOXARIFE" => $cdalmoxarife,
            "CDLOCALESTOQ" => $cdlocalestoq,
            "NRLOTEESTQ" => $nrloteestq,
            "NRSUBLOTE" => $nrsublote,
            "CDFILIMOVI" => $cdfilimovi,
            "CDOPERADOR" => $cdoperador
        );
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $nrseqitem = $std->fetch();

   }
   
      public function updateItemProduto($cdfilial, $nrlanctonf, $cdprodemis, $vrunititem, $idconforitem, $dsprodemis, $vrdescitem, $nrseqitprod,
                                    $idatuvrestoq, $vrtotaitem, $cdproduto, $qtitemprod, $vrcontitem, $vrbrutitem, $cdfilimovi,
                                    $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $cdoperador) {
                                        
        $sql = "UPDATE ITEMPRODUTO 
                    SET 
                    VRDESCITEM = :VRDESCITEM,
                    VRTOTAITEM = :VRTOTAITEM,
                    QTITEMPROD = :QTITEMPROD,
                    VRCONTITEM = :VRCONTITEM,
                    VRBRUTITEM = :VRBRUTITEM
                    WHERE  CDFILIAL = :CDFILIAL
                            AND CDPRODEMIS = :CDPRODEMIS
                            AND NRLANCTONF = :NRLANCTONF
                            AND CDOPERADOR = :CDOPERADOR
                            AND NRSEQITPROD = :NRSEQITPROD";
        
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDPRODEMIS" => $cdprodemis,
            "NRLANCTONF" => $nrlanctonf,
            "CDOPERADOR" => $cdoperador,
            "NRSEQITPROD" => $nrseqitprod,
            "VRDESCITEM" => $vrdescitem,
            "VRTOTAITEM" => $vrtotaitem,
            "QTITEMPROD" => $qtitemprod,
            "VRCONTITEM" => $vrcontitem,
            "VRBRUTITEM" => $vrbrutitem,
            
        );
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);

   }
}