<?php

namespace Teknisa\DF\Modules\Entrada\Factories;

/* Bibliotecas */
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Fiscal\Entities\Arqxmlnfe;
use Teknisa\Model\Fiscal\Entities\Notadestinada;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\FileUploader\Server as fileUploader;

class Dfe71000SincronizacaoCTEFactory {
    
    const PRODUCTION_SERVER_URL = 'https://midia.teknisa.com/index.php';
    const TAMANHO_LOTENFE = 15;
    
    protected $entityManager;
    protected $fileUploader;
    protected $environment;

    public function __construct(EntityManager $entityManager, $environment, $fileUploader) {
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
        $this->fileUploader  = $fileUploader;
        set_time_limit(0);
    }
    
    public function maxNrNsu($nrinsjurpesq, $tpAmb) {
        $sql = "SELECT NVL(MAX(TO_NUMBER(NRNSU)), 0) AS MAXNRNSU
                    FROM SINCRONIZACAOCTE
                   WHERE NRINSJURPESQ = :P_NRINSJURPESQ
                     AND IDTPAMBIENTE = :P_IDTPAMBIENTE";
                     
        $params = ["P_NRINSJURPESQ"  => $nrinsjurpesq,
                   "P_IDTPAMBIENTE"  => $tpAmb];

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch()['MAXNRNSU'];
    }
    
    public function existeNotaCte($nsu, $cnpj, $tpamb) {
        $sql = "SELECT NRACESSOCTE
                  FROM SINCRONIZACAOCTE
                 WHERE NRNSU = :P_NRNSU
                   AND NRINSJURPESQ = :P_NRINSJURPESQ
                   AND IDTPAMBIENTE = :P_IDTPAMBIENTE";
        
        $params = ["P_NRNSU" => $nsu,
                   "P_NRINSJURPESQ" => $cnpj,
                   "P_IDTPAMBIENTE" => $tpamb];

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
    }
    
    public function incluirNotaCte($dadosCte) {
        $sql = "INSERT INTO SINCRONIZACAOCTE
                   (NRINSJURPESQ, NRNSU,        IDTPAMBIENTE, NRACESSOCTE,
                    NRINSJUREMIT, NMRAZSOCEMIT, NRINSJURREME, NMRAZSOCREME,
                    NRINSJUREXPE, NMRAZSOCEXPE, NRINSJURRECE, NMRAZSOCRECE,
                    NRINSJURDEST, NMRAZSOCDEST, IDTOMADOR,    NRINSJURTOMA,
                    NMRAZSOCTOMA, DTEMISSAO,    VRTOTALPREST, VRRECEBER,
                    CDSTATUS,     DSSTATUS,     DTSINCRONIZACAO, NRORG)
                VALUES
                    (:P_NRINSJURPESQ, :P_NRNSU,        :P_IDTPAMBIENTE, :P_NRACESSOCTE, 
                     :P_NRINSJUREMIT, :P_NMRAZSOCEMIT, :P_NRINSJURREME, :P_NMRAZSOCREME,
                     :P_NRINSJUREXPE, :P_NMRAZSOCEXPE, :P_NRINSJURRECE, :P_NMRAZSOCRECE,
                     :P_NRINSJURDEST, :P_NMRAZSOCDEST, :P_IDTOMADOR,    :P_NRINSJURTOMA,
                     :P_NMRAZSOCTOMA, :P_DTEMISSAO,    :P_VRTOTALPREST, :P_VRRECEBER,
                     :P_CDSTATUS,     :P_DSSTATUS,     SYSDATE,         :P_NRORG)";
        
        $params = [
            "P_NRINSJURPESQ" => $dadosCte['NRINSJURPESQ'],
            "P_NRNSU"        => $dadosCte['NSU'],
            "P_IDTPAMBIENTE" => $dadosCte['IDTPAMBIENTE'],
            "P_NRACESSOCTE"  => $dadosCte['NRACESSOCTE'],
            "P_NRINSJUREMIT" => $dadosCte['NRINSJUREMIT'],
            "P_NMRAZSOCEMIT" => $dadosCte['NMRAZSOCEMIT'],
            "P_NRINSJURREME" => $dadosCte['NRINSJURREME'],
            "P_NMRAZSOCREME" => $dadosCte['NMRAZSOCREME'],
            "P_NRINSJUREXPE" => $dadosCte['NRINSJUREXPE'],
            "P_NMRAZSOCEXPE" => $dadosCte['NMRAZSOCEXPE'],
            "P_NRINSJURRECE" => $dadosCte['NRINSJURRECE'],
            "P_NMRAZSOCRECE" => $dadosCte['NMRAZSOCRECE'],
            "P_NRINSJURDEST" => $dadosCte['NRINSJURDEST'],
            "P_NMRAZSOCDEST" => $dadosCte['NMRAZSOCDEST'],
            "P_IDTOMADOR"    => $dadosCte['IDTOMADOR'],
            "P_NRINSJURTOMA" => $dadosCte['NRINSJURTOMA'],
            "P_NMRAZSOCTOMA" => $dadosCte['NMRAZSOCTOMA'],
            "P_DTEMISSAO"    => $dadosCte['DTEMISSAO'],
            "P_VRTOTALPREST" => $dadosCte['VRTOTALPREST'],
            "P_VRRECEBER"    => $dadosCte['VRRECEBER'],
            "P_CDSTATUS"     => $dadosCte['CDSTATUS'],
            "P_DSSTATUS"     => $dadosCte['DSSTATUS'],
            "P_NRORG"        => $this->environment->getNrOrgTrab()];

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function atualizaNotaCte($dadosCte) {
        $sql = "UPDATE SINCRONIZACAOCTE
                   SET NRINSJURPESQ = :P_NRINSJURPESQ,
                	   IDTPAMBIENTE = :P_IDTPAMBIENTE,
                	   NRACESSOCTE  = :P_NRACESSOCTE,
                	   NRINSJUREMIT = :P_NRINSJUREMIT,
                	   NMRAZSOCEMIT = :P_NMRAZSOCEMIT,
                	   NRINSJURREME = :P_NRINSJURREME,
                	   NMRAZSOCREME = :P_NMRAZSOCREME,
                	   NRINSJUREXPE = :P_NRINSJUREXPE,
                	   NMRAZSOCEXPE = :P_NMRAZSOCEXPE,
                	   NRINSJURRECE = :P_NRINSJURRECE,
                	   NMRAZSOCRECE = :P_NMRAZSOCRECE,
                	   NRINSJURDEST = :P_NRINSJURDEST,
                	   NMRAZSOCDEST = :P_NMRAZSOCDEST,
                	   IDTOMADOR    = :P_IDTOMADOR,
                	   NRINSJURTOMA = :P_NRINSJURTOMA,
                	   NMRAZSOCTOMA = :P_NMRAZSOCTOMA,
                	   DTEMISSAO    = :P_DTEMISSAO,
                	   VRTOTALPREST = :P_VRTOTALPREST,
                	   VRRECEBER    = :P_VRRECEBER,
                	   CDSTATUS     = :P_CDSTATUS,
                	   DSSTATUS     = :P_DSSTATUS
                 WHERE NRNSU        = :P_NRNSU
                   AND NRINSJURPESQ = :P_NRINSJURPESQ
                   AND IDTPAMBIENTE = :P_IDTPAMBIENTE
                   AND NRORG        = :P_NRORG";
                
        $params = [
            "P_NRINSJURPESQ" => $dadosCte['NRINSJURPESQ'],
            "P_NRNSU"        => $dadosCte['NSU'],
            "P_IDTPAMBIENTE" => $dadosCte['IDTPAMBIENTE'],
            "P_NRACESSOCTE"  => $dadosCte['NRACESSOCTE'],
            "P_NRINSJUREMIT" => $dadosCte['NRINSJUREMIT'],
            "P_NMRAZSOCEMIT" => $dadosCte['NMRAZSOCEMIT'],
            "P_NRINSJURREME" => $dadosCte['NRINSJURREME'],
            "P_NMRAZSOCREME" => $dadosCte['NMRAZSOCREME'],
            "P_NRINSJUREXPE" => $dadosCte['NRINSJUREXPE'],
            "P_NMRAZSOCEXPE" => $dadosCte['NMRAZSOCEXPE'],
            "P_NRINSJURRECE" => $dadosCte['NRINSJURRECE'],
            "P_NMRAZSOCRECE" => $dadosCte['NMRAZSOCRECE'],
            "P_NRINSJURDEST" => $dadosCte['NRINSJURDEST'],
            "P_NMRAZSOCDEST" => $dadosCte['NMRAZSOCDEST'],
            "P_IDTOMADOR"    => $dadosCte['IDTOMADOR'],
            "P_NRINSJURTOMA" => $dadosCte['NRINSJURTOMA'],
            "P_NMRAZSOCTOMA" => $dadosCte['NMRAZSOCTOMA'],
            "P_DTEMISSAO"    => $dadosCte['DTEMISSAO'],
            "P_VRTOTALPREST" => $dadosCte['VRTOTALPREST'],
            "P_VRRECEBER"    => $dadosCte['VRRECEBER'],
            "P_CDSTATUS"     => $dadosCte['CDSTATUS'],
            "P_DSSTATUS"     => $dadosCte['DSSTATUS'],
            "P_NRORG"        => $this->environment->getNrOrgTrab()];

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function pesquisaCteAceitos($nracessocte) {
        $sql = "SELECT NRINSJURPESQ, NRNSU, IDTPAMBIENTE, NRACESSOCTE,
                	   NRINSJUREMIT, NMRAZSOCEMIT, NRINSJURREME, NMRAZSOCREME,
                	   NRINSJUREXPE, NMRAZSOCEXPE, NRINSJURRECE, NMRAZSOCRECE,
                	   NRINSJURDEST, NMRAZSOCDEST, IDTOMADOR, NRINSJURTOMA,
                	   NMRAZSOCTOMA, DTEMISSAO, VRTOTALPREST, VRRECEBER, NRORG
                  FROM SINCRONIZACAOCTE
                 WHERE NRACESSOCTE = :P_NRACESSOCTE
                   AND CDSTATUS = '100'";
        
        $params = ["P_NRACESSOCTE"  => $nracessocte];

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
		
    }
    
	public function atualizaCteAceitos($array) {
		    $sql = "UPDATE SINCRONIZACAOCTE
                       SET NRINSJUREMIT = :P_NRINSJUREMIT,
                     	   NMRAZSOCEMIT = :P_NMRAZSOCEMIT,
                    	   NRINSJURREME = :P_NRINSJURREME,
                    	   NMRAZSOCREME = :P_NMRAZSOCREME,
                    	   NRINSJUREXPE = :P_NRINSJUREXPE,
                    	   NMRAZSOCEXPE = :P_NMRAZSOCEXPE,
                    	   NRINSJURRECE = :P_NRINSJURRECE,
                    	   NMRAZSOCRECE = :P_NMRAZSOCRECE,
                    	   NRINSJURDEST = :P_NRINSJURDEST,
                    	   NMRAZSOCDEST = :P_NMRAZSOCDEST,
                    	   IDTOMADOR    = :P_IDTOMADOR,
                    	   NRINSJURTOMA = :P_NRINSJURTOMA,
                    	   NMRAZSOCTOMA = :P_NMRAZSOCTOMA,
                    	   DTEMISSAO    = :P_DTEMISSAO,
                    	   VRTOTALPREST = :P_VRTOTALPREST,
                    	   VRRECEBER    = :P_VRRECEBER
                     WHERE NRACESSOCTE  = :P_NRACESSOCTE
                       AND CDSTATUS    != '100'
                       AND IDTPAMBIENTE = :P_IDTPAMBIENTE
                       AND NRORG        = :P_NRORG";
		    
		    $params = [
            "P_NRINSJUREMIT" => $array['NRINSJUREMIT'],
            "P_NMRAZSOCEMIT" => $array['NMRAZSOCEMIT'],
            "P_NRINSJURREME" => $array['NRINSJURREME'],
            "P_NMRAZSOCREME" => $array['NMRAZSOCREME'],
            "P_NRINSJUREXPE" => $array['NRINSJUREXPE'],
            "P_NMRAZSOCEXPE" => $array['NMRAZSOCEXPE'],
            "P_NRINSJURRECE" => $array['NRINSJURRECE'],
            "P_NMRAZSOCRECE" => $array['NMRAZSOCRECE'],
            "P_NRINSJURDEST" => $array['NRINSJURDEST'],
            "P_NMRAZSOCDEST" => $array['NMRAZSOCDEST'],
            "P_IDTOMADOR"    => $array['IDTOMADOR'],
            "P_NRINSJURTOMA" => $array['NRINSJURTOMA'],
            "P_NMRAZSOCTOMA" => $array['NMRAZSOCTOMA'],
            "P_DTEMISSAO"    => $array['DTEMISSAO'],
            "P_VRTOTALPREST" => $array['VRTOTALPREST'],
            "P_VRRECEBER"    => $array['VRRECEBER'],
            "P_NRACESSOCTE"  => $array['NRACESSOCTE'],
            "P_IDTPAMBIENTE" => $array['IDTPAMBIENTE'],
            "P_NRORG"        => $array['NRORG']];
		
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getNotasDownloadAutomatico($cnpj, $tpAmb) {
        $sql = "SELECT N.NRINSJURPESQ, N.IDTPAMBIENTE, N.NRNSU, N.NRACESSOCTE, A.DSARQXMLNSUCTE
                  FROM SINCRONIZACAOCTE N JOIN ARQXMLNSUCTE A ON (N.NRINSJURPESQ = A.NRINSJURPESQ)
                                                             AND (N.IDTPAMBIENTE = A.IDTPAMBIENTE)
                                                             AND (N.NRNSU = A.NRNSU)
                 WHERE N.NRINSJURPESQ = :P_NRINSJURPESQ
                   AND N.IDTPAMBIENTE = :P_IDTPAMBIENTE
                   AND N.NRACESSOCTE IS NOT NULL
                   AND N.CDSTATUS = 100
                   AND A.IDCONSULTADO = 'N'
                   AND NOT EXISTS (SELECT NRACESSONFE
                                     FROM NOTAFISCAL NF
                                    WHERE N.NRACESSOCTE = NF.NRACESSONFE
                                      AND NF.IDENTRSAIDOP = 'E')
                   AND NOT EXISTS (SELECT NRACESSONFE
                                     FROM INTG_NOTAFISCAL INF
                                    WHERE N.NRACESSOCTE = INF.NRACESSONFE)
                   AND NOT EXISTS (SELECT NRINSJURPESQ, IDTPAMBIENTE, NRNSU
                                     FROM ARQXMLNSUCTE ARQ
                                    WHERE N.NRINSJURPESQ = ARQ.NRINSJURPESQ
                                      AND N.IDTPAMBIENTE = ARQ.IDTPAMBIENTE
                                      AND N.NRNSU        = ARQ.NRNSU
                                      AND ARQ.CDSERVICO = 110111)";
        $params = [
            "P_NRINSJURPESQ" => $cnpj,
            "P_IDTPAMBIENTE" => $tpAmb];

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    
    public function atualizaCTeBaixado($nrinsjurpesq, $tpAmb, $nsu, $nrorg) {
        $sql = "UPDATE ARQXMLNSUCTE
                   SET IDCONSULTADO     = 'S'
                 WHERE NRINSJURPESQ     = :P_NRINSJURPESQ
                   AND IDTPAMBIENTE     = :P_IDTPAMBIENTE
                   AND TO_NUMBER(NRNSU) = TO_NUMBER(:P_NRNSU)
                   AND NRORG            = :P_NRORG";
                
        $params = [
            "P_NRINSJURPESQ" => $nrinsjurpesq,
            "P_IDTPAMBIENTE" => $tpAmb,
            "P_NRNSU"        => $nsu,
            "P_NRORG"        => $nrorg];
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);    
    }
    
    
    public function getXMLCTe($cnpj, $tpAmb, $nsu) {
        $sql = "SELECT DSARQXMLNSUCTE
                  FROM ARQXMLNSUCTE
                 WHERE NRINSJURPESQ = :P_NRINSJURPESQ
                   AND IDTPAMBIENTE = :P_IDTPAMBIENTE
                   AND NRNSU = :P_NRNSU      
                   AND DSARQXMLNSUCTE IS NOT NULL";
        $params = [
            "P_NRINSJURPESQ" => $cnpj,
            "P_IDTPAMBIENTE" => $tpAmb,
            "P_NRNSU" => $nsu];
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
    }
    
    public function getNotaSincronizada($nrAcessoNfe) {
        $sql = "SELECT NF.CDFILIAL,
                       NF.NRLANCTONF,
                       NF.NRACESSONFE,
                       NF.CDINSCESTA
                  FROM NOTAFISCAL NF
                 WHERE NF.NRACESSONFE = :P_NRACESSONFE
                   AND NF.IDENTRSAIDOP = 'E'";
         
        $params = ["P_NRACESSONFE"  => $nrAcessoNfe];
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function somenteXML($nrAcessoNfe) {
        $sql = "SELECT NRACESSONFE
                  FROM INTG_NOTAFISCAL
                 WHERE NRACESSONFE = :P_NRACESSONFE";
       
        $params = ["P_NRACESSONFE"  => $nrAcessoNfe];
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function getDiretorioIntegracao() {
        $sql = "SELECT DSDIRIMPARQXML FROM PARAMINTEGRA";
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetchAll(); 
        
    }
    
    public function getCNPJCertificado($ie, $cdempresa) {
        $sql = "SELECT NRINSJURIEST FROM INSCRESTAD WHERE CDEMPRESA = :CDEMPRESA AND CDINSCESTA = :CDINSCESTA";
        
        $params = ["CDEMPRESA"  => $cdempresa,
                   "CDINSCESTA" => $ie];
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch()['NRINSJURIEST'];
    }
    
    public function getCertificado($ie, $cnpj) {
        $sql = "SELECT DSDIRCERTIFICADO, CDSENHACERTIF FROM INSCRESTAD WHERE NRINSJURIEST = :CNPJ AND CDINSCESTA = :IE";
        
        $params = ["CNPJ" => $cnpj, "IE" => $ie];
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }

    /**
     * zGravaFile
     * Grava os dados no diretorio das NFe
     *
     * @param  string $tpAmb     ambiente
     * @param  string $filename  nome do arquivo
     * @param  string $data      dados a serem salvos
     * @param  string $subFolder
     * @param  string $chNFe
     * @throws Exception\RuntimeException
     */
    public function zGravaFile(
        $tpAmb = '2',
        $filename = '',
        $data = '',
        $subFolder = 'temporarias',
        $chNFe = ''
    ) {
        $tmpFile = sys_get_temp_dir().DIRECTORY_SEPARATOR.'xmlNota'.uniqid().'.xml';
        file_put_contents($tmpFile, $data);
        $file = file_get_contents($tmpFile);
        $file    = base64_encode($file);

        $file = 'data: '.mime_content_type($tmpFile).';base64,'.$file;
        $lastModify = filectime($tmpFile);

        $file = [
            "name" => $filename, 
            "lastModified" => $lastModify,
            "lastModifiedDate" => '',
            "webkitRelativePath" => '',
            "type" => '',
            "size"  => filesize($tmpFile),
            "b64File" => $file];
            
        $row['XML']    = [];
        $row['XML'][0] = $file;
        
        $subFolder .= date("Ym");
        $this->fileUploader->uploadFilesInRow($row, 'XML', $subFolder);
        unlink($tmpFile);
        
        $fileServerLink = $row['XML'][0]['path'];
        
        $subFolder = ($tpAmb == 1) ? 'producao/'.$subFolder : 'homologacao/'.$subFolder;
        
        $this->insertIntoRelacFileServer($chNFe, $subFolder, $fileServerLink);
        
        return $row['XML'][0]['path'];
    }
    
    private function insertIntoRelacFileServer($chNFe, $diretorio, $fileServerLink) {
        $sql = "INSERT
                INTO RELACFILESERVER
                   (CDRELAC,
                    DIRETORIO,
                    FILESERVERLINK,
                    NRORG)
                  VALUES
                   (:CDRELAC,
                    :DIRETORIO,
                    :FILESERVERLINK,
                    :NRORG)";
        
        $params = [
            "CDRELAC"        => $chNFe,
            "DIRETORIO"      => $diretorio,
            "FILESERVERLINK" => $fileServerLink,
            "NRORG"          => $this->environment->getNrOrgTrab()];

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function existeNotaRelac($chave) {
        $sql = "SELECT CDRELAC FROM RELACFILESERVER WHERE CDRELAC = :NRACESSONFE";
        
        $params = ["NRACESSONFE" => $chave];
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();
    }
    
    public function existeXmlNsuCTe($nrinsjurpesq, $tpamb, $nrnsu) {
        $sql = "SELECT NRNSU
                  FROM ARQXMLNSUCTE
                 WHERE NRINSJURPESQ = :P_NRINSJURPESQ
                   AND IDTPAMBIENTE = :P_IDTPAMBIENTE
                   AND NRNSU        = :P_NRNSU";
                
        $params = [
            "P_NRINSJURPESQ" => $nrinsjurpesq,
            "P_IDTPAMBIENTE" => $tpamb,
            "P_NRNSU"        => $nrnsu];
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();  
    }
    
    public function insereXmlNsuCTe($nrinsjurpesq, $tpamb, $nrnsu, $xml, $cdservico, $dsservico, $cdoperador, $nrorg) {
        $sql = "INSERT INTO ARQXMLNSUCTE
                    (NRINSJURPESQ, IDTPAMBIENTE, NRNSU, NRORG, NRORGINCLUSAO, CDOPERINCLUSAO, CDSERVICO, DSSERVICO, IDCONSULTADO, DSARQXMLNSUCTE)
                VALUES
                    (:P_NRINSJURPESQ, :P_IDTPAMBIENTE, :P_NRNSU, :P_NRORG, :P_NRORG, :P_CDOPERINCLUSAO, :P_CDSERVICO, :P_DSSERVICO, 'N', :P_DSARQXMLNSUCTE)";
                
        $params = [
            "P_NRINSJURPESQ"   => $nrinsjurpesq,
            "P_IDTPAMBIENTE"   => $tpamb,
            "P_NRNSU"          => $nrnsu,
            "P_CDSERVICO"      => $cdservico,
            "P_DSSERVICO"      => $dsservico,
            "P_DSARQXMLNSUCTE" => $xml,
            "P_CDOPERINCLUSAO" => $cdoperador,
            "P_NRORG"          => $nrorg];
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $newFileUploader) {
        if($useVpd)    $this->entityManager = $newEntityManager;
        if($nrorg > 1) $this->fileUploader  = $newFileUploader;
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
    }
    
    public function atualizaServDesac($nrinsjurpesq, $nsu, $idtpambiente, $nracessocte) {
        $sql = "UPDATE SINCRONIZACAOCTE
                   SET CDSTATUS     = :P_CDSTATUS,
                       DSSTATUS     = :P_DSSTATUS
                 WHERE NRINSJURPESQ = :P_NRINSJURPESQ
                   AND NRNSU        = :P_NRNSU
                   AND IDTPAMBIENTE = :P_IDTPAMBIENTE
                   AND NRACESSOCTE  = :P_NRACESSOCTE
                   AND CDSTATUS = 100
                   AND NRINSJURPESQ = NRINSJURTOMA";
        
        $params = [
            "P_CDSTATUS"        => '000', 
            "P_DSSTATUS"        => 'Serviço em Desacordo',
            "P_NRINSJURPESQ"    => $nrinsjurpesq,
            "P_NRNSU"           => $nsu,
            "P_IDTPAMBIENTE"    => $idtpambiente,
            "P_NRACESSOCTE"     => $nracessocte];    
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function maxEventoServDesac() {
        $sql = "SELECT NVL(MAX(NREVENTO), 0) AS NREVENTO FROM ARQXMLNFEVENTO";
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetch()['NREVENTO'];
    }
    
    public function insereXmlServDesac($nrevento, $nracessocte, $xml, $nrorg) {
        $sql = "INSERT INTO ARQXMLNFEVENTO
                    (NREVENTO, NRACESSONFE, IDTIPOARQXML, DSARQXMLNFE, NRORG)
                VALUES
                    (:P_NREVENTO, :P_NRACESSOCTE, :P_IDTIPOARQXML, :P_DSARQXMLNFE, :P_NRORG)";
                
        $params = [
            "P_NREVENTO"      => ($nrevento + 1),
            "P_NRACESSOCTE"   => $nracessocte,
            "P_IDTIPOARQXML"  => '610110',
            "P_DSARQXMLNFE"   => $xml,
            "P_NRORG"         => $nrorg];
            
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
}