<?php

namespace Teknisa\DF\Modules\Saida\Services;

use Teknisa\DF\Modules\Saida\Factories\Dfs61000TransmissaoNfseFactory;
use Geral\Util\DateUtil;


class Dfs61000TransmissaoNfseService {
    protected $transmissaoFactory;
    
    public function __construct(Dfs61000TransmissaoNfseFactory $transmissaoFactory) {
        $this->transmissaoFactory = $transmissaoFactory;
    }
    
    public function getCdReduCont($cdempresa, $cdcliente, $cdfilial, $nrlanctonf, $nrorg) {
        $cdreducont = $this->transmissaoFactory->getCdReduCont($cdempresa, $cdcliente, $nrorg);
        if(isset($cdreducont[0])) {
            if($cdreducont[0]["CDREDUCONT"]) return $cdreducont[0]["CDREDUCONT"];
        }
        $cdreducont = $this->transmissaoFactory->getCdReduCont1($cdfilial, $nrlanctonf, $nrorg);
        if(isset($cdreducont[0])) {
            if($cdreducont[0]["CDREDUCONT"]) return $cdreducont[0]["CDREDUCONT"];
        }
        $cdreducont = $this->transmissaoFactory->getCdReduCont2($cdempresa, $nrorg);
        return ($cdreducont[0]["CDREDUCONT"]) ? $cdreducont[0]["CDREDUCONT"] : NULL ;
    }
    
    public function getCarteira($cdempresa, $cdreducont, $nrorg) {
        $carteira = $this->transmissaoFactory->getCarteira($cdempresa, $cdreducont, $nrorg);
        return $carteira;
    }
    
    public function getCedente($cdreducont, $nrorg) {
        $cedente = $this->transmissaoFactory->getCedente($cdreducont, $nrorg);
        return $cedente;
    }
    
    public function getXmlNotas($stringNota, $cdfilial, $nrorg) {
        $xmls = $this->transmissaoFactory->getXmlNotas($stringNota, $cdfilial, $nrorg);
        return $xmls;
    }
    
    public function getLoteNotas($rows) {
        $cdempresa = $rows[0]["CDEMPRESA"];
        $sql = "(";
        foreach($rows as $row) {
            $nrlanctonf = $row['NRLANCTONF'];
            $sql.="'$nrlanctonf',";
        }
        $sql = substr($sql,0,-1);
        $sql.=")";
        return $this->transmissaoFactory->getLoteNotas($sql, $cdempresa);
    }
    
    public function updateDataEmissao($cdfilial , $nrlanctonf, $nrorg, $dtentrsaid){
        $dtatual =  $this->formataData(DateUtil::getDataAtual(), true);
        $dtatual= DateUtil::truncateString($dtatual);
        
        $dtatual = strtotime($dtatual) > strtotime($dtentrsaid) ? $dtentrsaid : $dtatual;
        return $this->transmissaoFactory->updateDataEmissao($cdfilial , $nrlanctonf, $nrorg,$dtatual);
    }
    
     public function formataData($data, $trunc = true){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        
        if($trunc)
            $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function updateStatus($cdfilial , $nrlanctonf, $nrorg){
        return $this->transmissaoFactory->updateStatus($cdfilial , $nrlanctonf, $nrorg);
    }
    
    public function getContareceber($cdfilial , $nrlanctonf, $nrorg){
        return $this->transmissaoFactory->getContareceber($cdfilial , $nrlanctonf, $nrorg);
    }
    
}